/**
 * \file gre/geolock.h
 * \brief GRE_GEOLOCKSETTINGS class definitions
 *
 * \if NODOC
 * $Id: geolock.h_v 1.8 2013/02/15 20:51:33 mju Exp $
 *
 * $Log: geolock.h_v $
 * Revision 1.8  2013/02/15 20:51:33  mju
 * Fix dox hdr.
 *
 * Revision 1.7  2006/09/15 15:29:45  mju
 * Serializer changes.
 *
 * Revision 1.6  2005/10/17 20:38:57  mju
 * Use gre namespace.
 *
 * Revision 1.5  2005/10/05 14:08:18  mju
 * Fix comment.
 *
 * Revision 1.4  2003/09/15 13:48:59  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.3  2003/07/30 15:40:20  mju
 * Go back to old non-namespace name as causes other problems.
 *
 * Revision 1.2  2003/07/30 14:53:14  mju
 * Ignore private section.
 * Put in GRE namespace with typedef for old name.
 *
 * Revision 1.1  2002/12/16 15:07:05  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_GEOLOCK_H
#define	INC_GRE_GEOLOCK_H

#ifndef INC_MI32_SERIALIZERBASE_H
	#include <mi32/serializerbase.h>
#endif

namespace GRE {

//! Settings for geo-locking, used by GRE_VIEW.
class GEOLOCKSETTINGS {
	public:

		enum MODE {
			MODE_Scale =				0x01,	//!< Lock by scale
			MODE_Extent =				0x02,	//!< Lock by extent
			MODE_ScaleAndExtent =	0x03,	//!< Lock by scale and extent
			};

		class FORM;								//!< User interface 'form'
		class DLG;								//!< User interface 'dialog'

		//! Get serialization item definition array.
		static const SERIALIZERITEM* SerialGetItemDef (
			);

		//! Constructor.
		GEOLOCKSETTINGS (
			): m_Mode(MODE_ScaleAndExtent), m_Scale(1) { }

		//! Get geolock mode.
		MODE GetMode (
			) const { return (m_Mode); }

		//! Get relative scale.
		double GetRelativeScale (
			) const { return (m_Scale); }

		//! Set geolock mode.
		void SetMode (
			MODE mode
			) { m_Mode = mode; }

		//! Set relative scale.
		void SetRelativeScale (
			double scale
			) { m_Scale = scale; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		MODE m_Mode;
		double m_Scale;

		#endif //!< GENERATING_DOXYGEN_OUTPUT
	};

}	// End namespace GRE

#endif	// INC_GRE_GEOLOCK_H
