/**
 * \file gre/legendimagesample.h
 * \brief GRE::LEGEND_IMAGESAMPLE definitions.
 *
 * \if NODOC
 * $Id: legendimagesample.h_v 1.3 2007/07/02 22:33:12 mju Exp $
 *
 * $Log: legendimagesample.h_v $
 * Revision 1.3  2007/07/02 22:33:12  mju
 * Add addImage.
 *
 * Revision 1.2  2007/06/08 17:46:33  mju
 * Add setThumbnails.
 *
 * Revision 1.1  2006/11/15 22:16:05  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_GRE_LEGENDIMAGESAMPLE_H
#define	INC_GRE_LEGENDIMAGESAMPLE_H

#ifndef	INC_GRE_LEGEND_H
	#include <gre/legend.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
namespace SPATMOD {
	namespace IMAGE {
		class STAGE;
		}
	}
namespace MGD {
	class DEVICE;
	}
#endif

namespace GRE {

class LAYER_IMAGE;

//===================================================================================================================

//! Image Sample legend subclass.
class LEGEND_IMAGESAMPLE : public GRE::LEGEND {
	public:

		enum LOCATION {
			LOCATION_TopLeft =		0,
			LOCATION_TopCenter =		1,
			LOCATION_TopRight =		2,
			LOCATION_CenterLeft =	3,
			LOCATION_Center =			4,
			LOCATION_CenterRight =	5,
			LOCATION_BottomLeft =	6,
			LOCATION_BottomCenter =	7,
			LOCATION_BottomRight =	8,
			};

		//! Individual item in legend.
		class ITEM : public GRE::LEGEND::ITEMBASE {
			public:
				//! Constructor.
				ITEM (
					LEGEND_IMAGESAMPLE& legend,
					MGD::DEVICE *& SampleMem,
					const MISTRING& label
					);

				//! Destructor.
				virtual ~ITEM (
					);

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				MGD::DEVICE *m_pSampleMem;

				// Overrides.
				virtual void v_Draw (const DRAWCONTEXT& dc, const LRECT2D& samplerect) const;
			#ifdef USE_MGUI
				virtual void v_TreeViewCustomDraw (GRE::CTRL_TREEVIEW& TreeView, MGUI::CUSTOMDRAW_TREE& CustomDraw);
			#endif

				ITEM (const ITEM&);
				ITEM& operator= (const ITEM&);
				#endif // GENERATING_DOXYGEN_OUTPUT
			};	// End of LAYER_IMAGE::LEGEND_IMAGESAMPLE::ITEM

		//! Construct legend for layer.
		explicit LEGEND_IMAGESAMPLE (
			LAYER_IMAGE *layer
			);

		//! Add item to legend.
		//! @return TRUE if item added, FALSE if all cells NULL in specified area, < 0 if error.
		int AddItem (
			SPATMOD::IMAGE::STAGE *pStageInput,
			const LPOINT2D& center,
			const MISTRING& label
			);											

		//! Add thumbnail to legend.
		//! @return TRUE if item added, FALSE if all cells NULL in specified area, < 0 if error.
		int AddThumbnail (
			SPATMOD::IMAGE::STAGE *pStageInput,
			const MISTRING& label
			);											

		//! Set item size.
		void SetItemSize (
			INT32 xsize,
			INT32 ysize
			) {
			if (m_ItemSize.x != xsize || m_ItemSize.y != ysize) {
				DeleteAllItems();
				m_ItemSize.x = xsize;
				m_ItemSize.y = ysize;
				}
			return;
			}

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		LPOINT2D m_ItemSize;

		ERRVALUE AddImage (SPATMOD::IMAGE::STAGE *pStageInput, const MISTRING& label);

		// Overrides.
		virtual void v_GetItemSizePixels (LPOINT2D& size) const;

		LEGEND_IMAGESAMPLE (const LEGEND_IMAGESAMPLE&);
		LEGEND_IMAGESAMPLE& operator= (const LEGEND_IMAGESAMPLE&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};	// End of LAYER_IMAGE::LEGEND_IMAGESAMPLE class.

//===================================================================================================================
}	// End namespace GRE

#endif	// INC_GRE_LEGENDIMAGESAMPLE_H
