/**
 * \file gre/lrpinmap.h
 * \brief Definitions for GRE::LAYER_PINMAP
 *
 * \if NODOC
 * $Id: lrpinmap.h_v 1.37 2012/06/13 15:01:32 mju Exp $
 *
 * $Log: lrpinmap.h_v $
 * Revision 1.37  2012/06/13 15:01:32  mju
 * Rmv v_appendPopupMenuItems.
 *
 * Revision 1.36  2011/10/10 17:48:33  mju
 * Move callDrawFilter to layer.
 *
 * Revision 1.34  2010/04/23 16:54:43  mju
 * Use unsignedset.
 *
 * Revision 1.33  2008/12/08 23:53:19  bcolombe
 * Added sml function to set crs in dispparm
 *
 * Revision 1.32  2008/08/11 13:58:26  mju
 * Add m_didValidateDb.
 *
 * Revision 1.31  2008/08/08 21:28:12  mju
 * Add v_getNumElementsDeleted.
 *
 * Revision 1.30  2008/06/12 17:39:00  bcolombe
 * Added GetPoints for getting multiple points at a time
 * Added Search to use internal quadtree to return elemnums inside a rect
 *
 * Revision 1.29  2008/05/15 17:55:02  mju
 * Chg v_constructLayerSection.
 *
 * Revision 1.28  2008/05/12 16:59:46  mju
 * Add parm to constructLayerSection.
 *
 * Revision 1.27  2008/03/17 21:33:27  mju
 * Chg v_getLayerView to v_getLayerSection.
 *
 * Revision 1.26  2008/01/02 22:45:20  mju
 * Add menuAddViewPopupItems.
 *
 * Revision 1.25  2007/08/02 21:25:23  dwilliss
 * added method to get SML class members
 *
 * Revision 1.24  2007/06/20 18:14:17  dwilliss
 * Added support for image viewer on right mouse button menu
 *
 * Revision 1.23  2007/06/20 16:02:06  vdronov
 * added 3d rendering
 *
 * Revision 1.21  2007/02/12 17:45:07  bcolombe
 * Added DrawFilter support
 *
 * Revision 1.20  2006/12/14 18:41:38  dwilliss
 * Added a private member to keep track of when we last checked extents
 *
 * Revision 1.19  2006/12/14 18:20:16  mju
 * Move numRecords to layer, not dispparm.
 * Change dispparm to expect objitem refers to table, make private.
 *
 * Revision 1.11  2006/01/10 22:20:57  mju
 * Add treeviewAddUpdateLegend.
 *
 * Revision 1.8  2005/10/19 20:34:09  mju
 * \endif
**/

#ifndef	INC_GRE_LRPINMAP_H
#define	INC_GRE_LRPINMAP_H

#ifndef	INC_GRE_LAYER_H
#include <gre/layer.h>
#endif

#ifndef	INC_GRE_LEGEND_H
#include <gre/legend.h>
#endif

#ifndef	INC_GRE_DATATIP_H
#include <gre/datatip.h>
#endif

#ifndef	INC_MI32_DATETIME_H
#include <mi32/datetime.h>
#endif

#ifndef	INC_MI32_LATLONFM_H
#include <mi32/latlonfm.h>
#endif


class QUADTREE;
namespace RVC {
	class DBTABLE;
	}
namespace MGUI {
	class DLG_SPATMOD_IMAGE_VIEW;
	class FORM_SPATMOD_IMAGE_VIEW;
	}

namespace GRE {
//===================================================================================================================

class LAYER_PINMAP : public GRE::LAYER {
	public:

		class DISPPARM : public GRE::LAYER::DISPPARM {
			public:

				class DLG;

				static ERRVALUE SerializerCB_StyleFileName (SERIALIZER&, const SERIALIZERITEM *, void *, SERIALIZERITEM::ACTION);

				RVC::OBJITEM StyleObjItem;
				INT16 xfield;							//!< -1 indicates set by query
				INT16 yfield;							//!< -1 indicates set by query
				INT16 zfield;							//!< -1 if by query, -2 if none
				INT16 labelfield;						//!< -1 if by query, -2 if none
				INT16 ThumbnailSymbolField;		//!< < 0 if not using thunbnail symbols
				UINT8 LabelPosn;						//!< Label position
				POINTSTYLE PointStyle;
				UINT16 SelectMode;					//!< Record selection mode - SELECTMODE_xxxxx...
				UINT16 StyleMode;						//!< Record style mode - STYLEMODE_xxxxx...
				MIUNICODE *QueryString;
				TEXTSTYLE LabelStyle;
				DMSTYPE LatLonFormat;
				UINT8 XYUnitCode;
				UINT8 ZUnitCode;
				GRE::DATATIPPARMS DataTip;				//!< DataTip parameters
				SPATREF::COORDREFSYS PointCRS;

				DISPPARM ();
				DISPPARM (
					const DISPPARM& rhs
					): GRE::LAYER::DISPPARM(rhs) { Copy(rhs); }
				virtual ~DISPPARM (
					) { Free(); }

				DISPPARM& operator= (const DISPPARM& rhs) {
					if (this != &rhs) {
						Free();
						Copy(rhs);
						}
					return (*this);
					}

				//! Get table object location.
				const RVC::OBJITEM& GetObjItemTable (
					) const { return (m_ObjItemTable); }

				ERRVALUE SetObjItemTable (
					const RVC::OBJITEM& ObjItem
					);

			protected:
				virtual const SERIALIZERITEM* SerialGetItemDef (SERIALIZER& serializer) const;

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT

				RVC::OBJITEM m_ObjItemTable;
				INT16 m_TableNumOld;								// Kept here so can load old dispparms and layouts

				void Copy (const DISPPARM&);
				void Free ();

				virtual ERRVALUE v_SetObjItem (const RVC::OBJITEM& objitem, bool replace);
				virtual ERRVALUE v_Validate ();

				friend class LAYER_PINMAP;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};

		//! Register this layer type
		static void RegisterType (
			);

		//! Install SML class, called by internal SML function.
		static void SMLClassInstall (
			SML::CONTEXT *context							//!< SML context
			);

		static const SML::CLASSMEMBER* GetSMLClassMembers (
			);

		//! Construct layer and insert into list
		explicit LAYER_PINMAP (
			GRE::GROUP *group,										//!< Group to contain layer
			CREATEFLAGS createflags = CREATEFLAG_None,	//!< Creation flags
			GRE::LISTPOS listpos = GRE::LISTPOS_Last,		//!< Position in list
			GRE::LAYER *reflayer = 0								//!< Reference layer for insertion
			);

		virtual ~LAYER_PINMAP ();

		//! Retrieve layer-specific display parameters.
		const DISPPARM& GetDispParm (
			) const {
			return (m_dispparm);
			}

		//! Get point coordinates for specified "element" (record number).
		ERRVALUE GetPoint (
			INT32 ElemNum,									//!< Element record number in table
			DPOINT3D& point								//!< Point coordinates returned
			);

		//! Get point coordinates for specified "elements" (record number).
		ERRVALUE GetPoints (
			const SIMPLE_ARRAY<INT32>& ElemNums,					//!< Element record number in table
			POLYLINE& points								//!< Point coordinates returned
			);

		ERRVALUE Search (
			const DRECT2D& SearchExtents,
			SIMPLE_ARRAY<INT32>& ItemList
			);

		//! Set layer-specific display parameters.
		//!
		//! This is the recommended method for changing how an existing layer is to be
		//! rendered.  Validation will be performed on the specified parameters and
		//! adjustments made to the layer's copy if necessary.  Use of this method
		//! usually result in a ChangeBegin / ChangeEnd message pair, and will also
		//! generate a ChangeObject message if the spatial object used in the layer
		//! is changed.
		ERRVALUE SetDispParm (
			const DISPPARM& dispparm					//!< Parameters to set
			);

		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIVDLGDATA;
		class PRIVDRAWDATA;
		#endif

	protected:

		DISPPARM m_dispparm;								//!< Layer-specific display parameters

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		class DBOBSERVER;
		typedef LAYER_PINMAP THISCLASS;

		friend class PRIVDRAWDATA;
		friend class PRIVDLGDATA;
		friend class DBOBSERVER;

		GRE::LEGEND_POINT m_Legend;
		COMPINFO m_RecCompInfo;
		QUADTREE *m_SearchTree;
		INT32 m_LastNumRecords;
		INT32 m_SearchNumAdded;
		DBOBSERVER *m_pDBObserver;
		RVC::DBTABLE *m_pObjTable;
		DATETIME m_LastMod;		// LastMod time of the table when we last checked extents
		bool m_DidValidateDb;
#ifdef USE_MGUI
		INT32 m_RMBMenuRecordNum;	// Record number closest to cursor on last Right Mouse Button menu
		INT32 m_RMBMenuFieldNum;	// Filed number for photo viewer
		MGUI::DLG_SPATMOD_IMAGE_VIEW* m_pDlgImageView;	// Image viewer	// XXX This really needs to be on the view, but v_AppendPopupMenu doesn't give us the view
#endif
		GRE::GEOMETRIC3D *m_pGeometric3D;

		static void LegendMsgHandler (GRAPHICTREEITEM_MSG *msg);
		static void SMLSET_CRS (SML::CONTEXT*, SML::CLASSMEMBER*, void*, SML::ARG*);
		static void SMLGET_StyleByScript (SML::CONTEXT*, SML::CLASSMEMBER*, void*, SMLARG*);
		static void SMLSET_Field (SML::CONTEXT*, SML::CLASSMEMBER*, void*, SMLARG*);
		static void SMLSET_StyleByScript (SML::CONTEXT*, SML::CLASSMEMBER*, void*, SMLARG*);

		void ExtendRegion (const DPOINT3D& point);
		void OnRecordChangeBegin (INT32 RecNum);
		void OnRecordChangeEnd (INT32 RecNum);
#ifdef USE_MGUI
		void OnMenuPopupImage (MGUI::ID id, bool ToggleOn);
		void OnPrevPhoto (MGUI::FORM_SPATMOD_IMAGE_VIEW*);
		void OnNextPhoto (MGUI::FORM_SPATMOD_IMAGE_VIEW*);
		void PopupPhoto ();
#endif
		ERRVALUE OpenPrivateObject ();
		ERRVALUE SetupObserver ();

		// temporary methods may go away ***************
		bool NewFeaturesFilter (LAYERDC& layerdc);
		bool IsModeWireFrame (LAYERDC& layerdc);
		bool IsSurfaceFlat (LAYERDC& layerdc);
		// **********************************************

		ERRVALUE Draw2D (LAYERDC& layerdc);
		ERRVALUE Draw3D (LAYERDC& layerdc);

		//! Overrides from GRE::LAYER.
		virtual ERRVALUE v_AssignObject (const RVC::OBJITEM& objitem, MDLGPARENT dlgparent);
		virtual bool v_CanShowDetails () const;
		virtual GRE::LAYERSECTION* v_ConstructLayerSection (GRE::VIEW* view);
	#ifdef USE_MGUI
		virtual int v_ControlPanel (MDLGPARENT dlgparent,UINT32 flags = 0);
	#endif
		virtual GRE::LAYER* v_Copy (GRE::GROUP *group) const;
		virtual const GRE::DATATIPPARMS* v_DataTipGetParms (int component) const;
		virtual int v_DataTipGetString (ELEMTYPE ElemType, ELEMNUM ElemNum, MISTRING& string, SPATMOD::IMAGE::STAGE_ARRAY& images);
		virtual ERRVALUE v_DataTipSetParms (int component, const GRE::DATATIPPARMS& DataTipParms);
		virtual ERRVALUE v_Draw (GRE::LAYERDC& layerdc);
		virtual ERRVALUE v_DrawElementLow (GRE::LAYERDC& layerdc, ELEMTYPE ElemType, ELEMNUM ElemNum, const void *style);
		virtual ERRVALUE v_DrawElementsLow (LAYERDC& layerdc, ELEMTYPE ElemType, const UNSIGNEDSET *pElemSet, const void *style);
		virtual ELEMNUM v_FindClosestElement (const DPOINT2D& point, double searchdist, double *distret, int component, const UNSIGNEDSET *pSearchSet, GRE::FINDFLAGS findflags);
		virtual int v_GetComponent (ELEMTYPE ElemType) const;
		virtual MICONSTRUCT* v_GetComponentIcon (int component) const;
		virtual COMPINFO* v_GetComponentInfo (int component);
		virtual MISTRING v_GetComponentName (int component) const;
		virtual const void* v_GetElemDefaultStyle (ELEMTYPE ElemType) const;
		virtual UINT16 v_GetElemDrawSelectionMode (ELEMTYPE ElemType) const;
		virtual ERRVALUE v_GetElemExtents (ELEMTYPE ElemType, ELEMNUM ElemNum, DRECT3D& Extents);
		virtual ELEMTYPE v_GetElemType (int component) const;
		virtual bool v_GetLabelColor(GRE::LAYERDC&, const GRE::LABELPLACE&, COLOR&);
		virtual const GRE::LEGEND* v_GetLegend (ELEMTYPE ElemType);
		virtual INT64 v_GetNumElements (ELEMTYPE ElemType);
		virtual INT64 v_GetNumElementsDeleted (ELEMTYPE ElemType);
		virtual const RVC::OBJITEM& v_GetObjItem (int component) const;
		virtual ERRVALUE v_GetStyleInfo (ELEMTYPE ElemType, RVC::OBJITEM& ObjItem, RVC::OBJECTNAME& TableName);
		virtual bool v_HasLegend () const;
		virtual int v_HighlightElementsByRegion (ELEMTYPE ElemType, const REGION2D& Region, TESTCRITERIA RegionTest, GRE::VIEW *firstview, GRE::DRAWFLAGS drawflags);
	#ifdef USE_MGUI
		virtual void v_MenuAddViewPopupItems (MGUI::MENU& menu, const DPOINT2D *point, GRE::VIEW *view);
		virtual ERRVALUE v_PromptObject (bool OpenCtrlPanel, MDLGPARENT dlgparent);
	#endif
		virtual const char* v_SerialGetTagName () const;
		virtual ERRVALUE v_SerialRead (SERIALIZER& serializer);
		virtual void v_SetElemDrawSelectionMode (ELEMTYPE ElemType, UINT16 SelectMode);
		virtual void v_SetLegendSampleFlags (MGD::STYLESAMPLEFLAGS SampleFlags);
		virtual ERRVALUE v_SetStyleObjItem (const RVC::OBJITEM& ObjItem);
		virtual int v_TestElementByRegion (ELEMTYPE ElemType, ELEMNUM ElemNum, const REGION2D& Region, TESTCRITERIA TestMode);
	#ifdef USE_MGUI
		virtual MGUI::CTRL_TREE::ITEMHND v_TreeViewAddUpdateLegend (GRE::CTRL_TREEVIEW& TreeView, MGUI::CTRL_TREE::ITEMHND hndParent, MGUI::CTRL_TREE::ITEMHND hndInsertAfter);
		virtual void v_TreeViewAddUpdateDetails (GRE::CTRL_TREEVIEW& TreeView, MGUI::CTRL_TREE::ITEMHND hndParent, MGUI::CTRL_TREE::ITEMHND hndInsertAfter);
	#endif
		virtual int v_UpdateExtents (bool forceupdate);

		LAYER_PINMAP (const LAYER_PINMAP&);
		LAYER_PINMAP& operator= (const LAYER_PINMAP&);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================
} // End namespace GRE



#endif	// INC_GRE_LRPINMAP_H

