/**
 * \file gre/lrtileset.h
 * \brief GRE LAYER_TILESET definitions
 *
 * \if NODOC
 * $Id: lrtileset.h_v 1.19 2012/01/20 15:18:29 mju Exp $
 *
 * $Log: lrtileset.h_v $
 * Revision 1.19  2012/01/20 15:18:29  mju
 * Remove legacy terrain layer.
 *
 * Revision 1.18  2011/02/07 23:22:44  bcolombe
 * Add v_DatyatipGetObjectNamesString
 *
 * Revision 1.17  2011/02/04 18:28:06  bcolombe
 * No flag in ctor for local, use objitem
 *
 * Revision 1.16  2011/02/01 16:33:32  bcolombe
 * Add v_CanRender
 *
 * Revision 1.15  2010/11/19 22:57:17  bcolombe
 * Remove use of DLG_GETOBJECT::ITEM
 *
 * Revision 1.14  2010/11/19 17:14:33  bcolombe
 * Add option to show datatip
 *
 * Revision 1.13  2010/08/19 23:08:53  bcolombe
 * Add v_AssignItem
 *
 * Revision 1.12  2010/08/16 16:14:17  bcolombe
 * Only include dlggetobjectitem.h if mgui
 *
 * Revision 1.11  2010/08/11 22:04:25  bcolombe
 * Added option to construct layer from DLG_GETOBJECT::ITEM
 *
 * Revision 1.10  2010/06/10 21:26:01  bcolombe
 * Add v_MenuAddLayerItems
 *
 * Revision 1.9  2010/03/02 21:52:19  bcolombe
 * Added m_ZoomLevelArray
 *
 * Revision 1.8  2009/11/06 18:28:26  mju
 * Add v_assignObject.
 *
 * Revision 1.7  2009/09/28 20:08:23  bcolombe
 * Added support for transparent colors
 *
 * Revision 1.6  2009/09/24 23:02:32  bcolombe
 * Added transparency and saturation
 *
 * Revision 1.5  2009/09/24 18:05:12  bcolombe
 * Add relief shading and enlargement method
 *
 * Revision 1.4  2009/09/23 22:50:43  bcolombe
 * Add layer controls
 *
 * Revision 1.3  2009/09/09 14:30:40  bcolombe
 * Added v_PromptObject
 *
 * Revision 1.2  2009/08/17 16:24:23  bcolombe
 * Added v_GetScaleMaxDetail
 *
 * Revision 1.1  2009/08/07 16:14:14  bcolombe
 * Initial revision
 *
 *
 * \endif
**/

#ifndef	INC_GRE_LRTILESET_H
#define	INC_GRE_LRTILESET_H

#ifndef	INC_GRE_LRIMAGE_H
#include <gre/lrimage.h>
#endif

#ifndef INC_SPATMOD_IMAGESOURCETILESET_H
#include <spatmod/imagesourcetileset.h>
#endif

#ifndef INC_SPATMOD_IMAGERESAMPLEMETHOD_H
#include <spatmod/imageresamplemethod.h>
#endif

#ifndef INC_MI32_RASTERFILTERPARMS_H
	#include <mi32/rasterfilterparms.h>
#endif

namespace GRE {
//===================================================================================================================

//! Tileset Layer class
class LAYER_TILESET : public GRE::LAYER_IMAGE {
	public:

		// LAYER_TILESET::DISPPARM
		class DISPPARM : public LAYER::DISPPARM {
			public:
				DISPPARM ();

				DISPPARM (
					const DISPPARM& rhs
					);

				virtual ~DISPPARM ();

				DISPPARM& operator= (
					const DISPPARM& rhs
					) { if (this != &rhs) { Copy(rhs); } return (*this); }

				bool GetBlackTransparent (
					) const { return (m_bBlackTransparent); }

				bool GetWhiteTransparent (
					) const { return (m_bWhiteTransparent); }

				bool GetDefinedTransparent (
					) const { return (m_bDefinedTransparent); }

				bool GetDoShading (
					) const { return (m_bDoShading); }

				const RVC::OBJITEM& GetObjItem (
					) const { return (m_objitem); }

				RASTER_FILTER_PARMS GetRasterFilterParms (
					) const { return (m_RasterFilterParms); }

				SPATMOD::IMAGE::RESAMPLEMETHOD GetResampleMethod (
					) const { return (m_ResampleMethod); }

				double GetSaturation (
					) const { return (m_Saturation); }

				bool GetShowDatatipTileInfo (
					) const { return (m_bShowDatatipTileInfo); }

				bool GetShowDatatipImageValue (
					) const { return (m_bShowDatatipImageValue); }

				bool GetShowLabels (
					) const { return (m_bShowLabels); }

				double GetTransparency (
					) const { return (m_Transparency); }

				//! Determine if has been validated.
				bool IsValidated (
					) const { return (m_IsValidated); }

				void SetBlackTransparent (
					bool BlackTransparent
					) { m_bBlackTransparent = BlackTransparent; }

				void SetWhiteTransparent (
					bool WhiteTransparent
					) { m_bWhiteTransparent = WhiteTransparent; }

				void SetDefinedTransparent (
					bool DefinedTransparent
					) { m_bDefinedTransparent = DefinedTransparent; }

				void SetDoShading (
					bool DoShading
					) { m_bDoShading = DoShading; }

				void SetObjItem (
					const RVC::OBJITEM& objitem
					) { m_objitem = objitem; }

				void SetRasterFilterParms (
					const RASTER_FILTER_PARMS& RasterFilterParms
					) { m_RasterFilterParms = RasterFilterParms; }

				void SetResampleMethod (
					SPATMOD::IMAGE::RESAMPLEMETHOD ResampleMethod
					) { m_ResampleMethod = ResampleMethod; }

				void SetSaturation (
					double Saturation
					) { m_Saturation = Saturation; }

				void SetShowDatatipTileInfo (
					bool ShowDatatipTileInfo
					) { m_bShowDatatipTileInfo = ShowDatatipTileInfo; }

				void SetShowDatatipImageValue (
					bool ShowDatatipImageValue
					) { m_bShowDatatipImageValue = ShowDatatipImageValue; }

				void SetShowLabels (
					bool ShowLabels
					) { m_bShowLabels = ShowLabels; }

				void SetTransparency (
					double Transparency
					) { m_Transparency = Transparency; }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				RVC::OBJITEM m_objitem;
				SPATMOD::IMAGE::RESAMPLEMETHOD m_ResampleMethod;
				RASTER_FILTER_PARMS m_RasterFilterParms;
				double m_Saturation;
				double m_Transparency;
				bool m_bDoShading;
				bool m_bShowDatatipTileInfo;
				bool m_bShowDatatipImageValue;
				bool m_bShowLabels;
				bool m_bBlackTransparent;
				bool m_bWhiteTransparent;
				bool m_bDefinedTransparent;
				bool m_IsValidated;

				void Copy (const DISPPARM&);

				// SERIALIZABLE overrides
				virtual const SERIALIZERITEM* SerialGetItemDef (SERIALIZER& serializer) const;
				virtual ERRVALUE SerialRead (SERIALIZER& serializer);
				virtual ERRVALUE SerialWrite (SERIALIZER& serializer, const char *tagname = 0) const;

				// LAYER::DISPPARM overrides
				virtual ERRVALUE v_Validate ();

				#endif // GENERATING_DOXYGEN_OUTPUT
			};	// End of LAYER_TILESET::DISPPARM

		//! Register this layer type
		static void RegisterType (
			);

		//! Method to install SML class, called by internal SML function.
		static void SMLClassInstall (
			SMLCONTEXT *context							//!< SML context
			);

		//! Construct layer and insert into list
		explicit LAYER_TILESET (
			GRE::GROUP *group,							//!< Group to contain layer
			CREATEFLAGS createflags = CREATEFLAG_None,	//!< Creation flags
			GRE::LISTPOS listpos = GRE::LISTPOS_Last,	//!< Position in list
			LAYER *reflayer = 0							//!< Reference layer for insertion
			);

		virtual ~LAYER_TILESET ();

		//! Retrieve layer-specific display parameters.
		//! @return current parameters
		const DISPPARM& GetDispParm (
			) const {
			return (m_dispparm);
			}

		//! Set layer-specific display parameters.
		void SetDispParm (
			const DISPPARM& dispparm					//!< Parameters to set
			) { m_dispparm = dispparm; }

	protected:

		DISPPARM m_dispparm;								//!< Layer-specific display parameters

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		typedef LAYER_IMAGE BASECLASS;
		
		class DLG;
		#ifdef USE_MGUI
			DLG *m_pDlgControlPanel;
		#endif

		SPATMOD::IMAGE::SOURCE_TILESET *m_pSource;
		SIMPLE_ARRAY<INT32> m_ZoomLevelArray;
		bool m_HasError;

		bool FilterDelegate (
			const SPATMOD::IMAGE::SAMPLEITERATOR *ittgt,
			const SPATMOD::IMAGE::SAMPLEITERATOR *itsrc,
			bool valid
			);

		ERRVALUE InsertSpatialFilter (
			SPATMOD::IMAGE::STAGE_AUTOPTR& StagePtr
			);

		ERRVALUE Draw2D (LAYERDC& layerdc);
		bool CanDoShading();

		#ifdef USE_MGUI
		void OnMenuTool (MGUI::ID, bool);
		#endif

		// Overrides from LAYER.
		virtual ERRVALUE v_AssignObject (const RVC::OBJITEM& objitem, MDLGPARENT dlgparent);
		#ifdef USE_MGUI
		virtual int v_ControlPanel (MDLGPARENT dlgparent, UINT32 flags = 0);
		virtual ERRVALUE v_PromptObject (bool OpenCtrlPanel, MDLGPARENT dlgparent);
		virtual void v_MenuAddLayerItems (MGUI::MENU& menu);
		#endif
		bool v_DataTipHasEnabled () const;
		virtual void v_DataTipGetObjectNamesString (const DPOINT2D& point, MISTRING& string, int indent, GRE::VIEW *view) const;
		int v_DataTipGetString (const DPOINT2D& point, MISTRING& string, SPATMOD::IMAGE::STAGE_ARRAY& images, GRE::VIEW *view);
		virtual bool v_CanRender () const;
		virtual LAYER* v_Copy (GRE::GROUP *group) const;
		virtual ERRVALUE v_Draw (LAYERDC& layerdc);
		virtual double v_GetScaleMaxDetail (const DPOINT2D& point, const SPATREF::COORDREFSYS& crs, double DisplayPixelSizeMM) const;
		virtual bool v_HasNullImageCells () const;
		virtual const char* v_SerialGetTagName () const;
		virtual void v_SetDftName (bool ReplaceExisting);
		virtual ERRVALUE v_UpdateExtents (bool forceupdate);
		
		LAYER_TILESET (const LAYER_TILESET&);
		LAYER_TILESET& operator= (const LAYER_TILESET&);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};	// End of LAYER_TILESET


//===================================================================================================================
}	// End namespace GRE

#endif	// INC_GRE_LRTILESET_H
