/**
 * \file gre/uishellview.h
 * \brief GRE::UI::SHELL_VIEW definitions
 *
 * \if NODOC
 * $Id: uishellview.h_v 1.5 2012/05/17 16:31:21 mju Exp $
 *
 * $Log: uishellview.h_v $
 * Revision 1.5  2012/05/17 16:31:21  mju
 * Add setMenuBarSubMenus.
 *
 * Revision 1.4  2011/12/20 18:43:39  mju
 * Add settings to ctor.
 *
 * Revision 1.3  2011/11/28 17:14:17  mju
 * Fix dox dependencies.
 *
 * Revision 1.2  2011/05/26 14:23:25  mju
 * Change to use 'viewable' instead of 'group'.
 * Add setDelegateAddTools.
 *
 * Revision 1.1  2008/01/17 17:19:07  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_UISHELLVIEW_H
#define	INC_GRE_UISHELLVIEW_H

#ifndef	INC_GRE_UIFORMVIEW_H
	#include <gre/uiformview.h>
#endif

#ifndef	INC_MGUI_TOPSHELL_H
	#include <mgui/topshell.h>
#endif

namespace GRE {
namespace UI {

//===================================================================================================================

//! Shell containing view form.
class SHELL_VIEW : public MGUI::TOPSHELL {
	public:

		//! Create and open shell for viewable.
		//! Shell will be set to self-destruct on close.
		//! Will be created with default menubar.
		static SHELL_VIEW* CreateAndOpen (
			GRE::VIEWABLE *viewable,
			INT16 CanvasWidth = 200,
			INT16 CanvasHeight = 200,
			GRE::UI::FORM_VIEW::CREATEFLAGS CreateFlags = FORM_VIEW::CREATEFLAG_None,
			const char *placementkeystr = 0
			);

		//! Construct for viewable.
		SHELL_VIEW (
			GRE::VIEWABLE *viewable,
			INT16 CanvasWidth = 200,
			INT16 CanvasHeight = 200,
			GRE::UI::FORM_VIEW::CREATEFLAGS CreateFlags = FORM_VIEW::CREATEFLAG_None,
			GRE::UI::FORM_VIEW::SETTINGS *pFormSettings = 0
			);

		//! Destructor.
		virtual ~SHELL_VIEW ();

		//! Get UI::FORM_VIEW instance.
		GRE::UI::FORM_VIEW& GetFormView ()
			{ return (m_FormView); }

		//! Set delegate to call to add extra tools.
		void SetDelegateAddTools (
			UI::FORM_VIEW::DELEGATE_ADDTOOLS delegate
			) { m_FormView.SetDelegateAddTools(delegate); }

		//! Set to self-destruct on close.
		//! Default is true if created via CreateAndOpen, otherwise false.
		void SetDestroySelfOnClose (
			bool DestroySelfOnClose = true
			) { m_DestroySelfOnClose = DestroySelfOnClose; }

		//! Set whether to destroy viewable on close.
		//! If not called viewable will not be destroyed.
		void SetDestroyViewableOnClose (
			bool DestroyViewableOnClose = true
			) { m_DestroyViewableOnClose = DestroyViewableOnClose; }

		//! Set which submenus will appear in menubar.
		//! By default, View, Tools, Options, Macros and HotKeys will be included.
		//! Must be called before Create().
		void SetMenuBarSubMenus (
			UI::FORM_VIEW::SUBMENUS submenus		//!< Which submenus to include
			) { m_MenuBarSubMenus = submenus; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		GRE::UI::FORM_VIEW m_FormView;
		GRE::UI::FORM_VIEW::CREATEFLAGS m_FormCreateFlags;
		GRE::UI::FORM_VIEW::SETTINGS *m_pFormSettings;
		UI::FORM_VIEW::SUBMENUS m_MenuBarSubMenus;
		INT16 m_CanvasWidth;							// View canvas width
		INT16 m_CanvasHeight;						// View canvas height
		bool m_DestroySelfOnClose;
		bool m_DestroyViewableOnClose;

		static void GRECB_View (const GRE::MSG *msg, void *vpShell);

		void OnGreMsg (const GRE::MSG& msg);

		// SHELL overrides.
		virtual int v_CreateContent ();
		virtual void v_OnClose ();
		virtual void v_OnCloseRequest ();
		virtual void v_OnOpen ();

		// Unimplemented.
		SHELL_VIEW (const SHELL_VIEW&);
		SHELL_VIEW& operator= (const SHELL_VIEW&);
		#endif
	};

//===================================================================================================================

}	// End namespace UI
}	// End namespace GRE

#endif	// INC_GRE_UISHELLVIEW_H
