/**
 * \file gre/viewscript.h
 * \brief GRE::VIEWSCRIPT and related class definitions
 *
 * \if NODOC
 * $Id: viewscript.h_v 1.28 2011/11/28 17:22:48 mju Exp $
 *
 * $Log: viewscript.h_v $
 * Revision 1.28  2011/11/28 17:22:48  mju
 * Fix dox dependencies.
 *
 * Revision 1.27  2010/11/10 16:53:15  bcolombe
 * Add IsEncrypted
 *
 * Revision 1.26  2010/11/09 22:43:53  bcolombe
 * Add Load that takes objitem
 *
 * Revision 1.25  2008/05/13 17:50:24  mju
 * Change context ctor to take tool pointer.
 *
 * Revision 1.24  2008/04/29 13:51:31  mju
 * Add gadget ptr to toolscript context ctor.
 *
 * Revision 1.23  2008/02/04 20:51:47  mju
 * Add getIconID and onCmdRun/Menu.
 *
 * Revision 1.22  2008/02/01 16:38:19  mju
 * Remove x-specific methods from macroscript::context.
 *
 * Revision 1.21  2008/01/25 15:36:10  mju
 * Move toolscript::context defn here.
 * Make macro/toolscriptlist class instead of typedef so can fwd-declare.
 *
 * Revision 1.18  2007/01/31 18:30:23  dwilliss
 * Added a usage parameter to EditScript method
 *
 * Revision 1.17  2006/10/10 13:47:47  mju
 * Use miconstruct, remove some inlines and micon hdr.
 *
 * Revision 1.1  2003/06/16 19:40:49  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_VIEWSCRIPT_H
#define	INC_GRE_VIEWSCRIPT_H

#ifndef	INC_GRE_BASE_H
	#include <gre/base.h>
#endif

#ifndef	INC_RVC_OBJITEM_H
	#include <rvc/objitem.h>
#endif

#ifndef	INC_MI32_SERIALIZABLE_H
	#include <mi32/serializable.h>
#endif

#ifndef	INC_MI32_SERIALIZERITEM_H
	#include <mi32/serializeritem.h>
#endif

#ifndef	INC_MI32_SMLBASE_H
	#include <mi32/smlbase.h>
#endif

#ifndef	INC_MI32_ICONID_H
	#include <mi32/iconid.h>
#endif

#ifndef	INC_MI32_MISTRINGLIST_H
	#include <mi32/mistringlist.h>
#endif

#ifndef	INC_MGUI_KEYSTATE_H
	#include <mgui/keystate.h>
#endif

#ifndef	INC_MGUI_POINTERBUTTON_H
	#include <mgui/pointerbutton.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
struct MICONSTRUCT;
namespace MGUI {
	class TOOL_BASE;
	}
namespace SML {
	class GUI_GADGET;
	}
#endif

namespace GRE {

//===================================================================================================================

class VIEWSCRIPT : public SERIALIZABLE {
	public:

		class CONTEXT;

		//! Default Constructor.
		VIEWSCRIPT ();

		//! Copy constructor.
		VIEWSCRIPT (
			const VIEWSCRIPT& rhs
			): SERIALIZABLE(rhs) { Copy(rhs); }

		//! Destructor.
		virtual ~VIEWSCRIPT ();

		//! Assignment operator.
		VIEWSCRIPT& operator= (
			const VIEWSCRIPT& rhs
			);

		virtual void IniWrite (
			const char* IniGroup,
			const char* IniBaseName
			) const;

		virtual int Load (
			const char* IniGroup = 0,
			const char* IniBaseName = 0
			);

		bool EditProperties (
			MDLGPARENT dlgparent,
			GRE::VIEWABLE *viewable,
			GRE::VIEW *view,
			bool CanChangeSet
			);

		//! @return TRUE if changed, FALSE if not, < 0 if error
		int EditScript (
			MDLGPARENT dlgparent,
			SMLCONTEXT* smlcontext,
			SMLUSAGE usage
			);

		const FILEPATH& GetFilePath (
			) const { return (m_ObjItemScript.GetFilePath()); }

		MICONSTRUCT* GetIcon (
			) const;

		const char* GetIconGroup (
			) const { return (m_IconGroup); }

		ICONID GetIconID (
			) const;

		const char* GetIconName (
			) const { return (m_IconName); }

		const MIUNICODE* GetName (
			) const { return (m_name); }

		const MISTRING& GetScriptString (
			) const { return (m_scriptstr); }

		bool GetShowIcon (
			) const { return (m_ShowIcon); }

		bool IsFromViewable (
			) const { return (m_FromViewable); }

		bool IsEncrypted (
			) const { return (m_bIsEncrypted); }

		ERRVALUE Load (
			const FILEPATH& filepath
			);

		ERRVALUE Load (
			const RVC::OBJITEM& objitem
			);

		bool SelectIcon (
			MDLGPARENT dlgparent
			);

		void SetFilePath (
			const FILEPATH& filepath
			) { m_ObjItemScript.SetFilePath(filepath); }

		void SetFromViewable (
			bool FromViewable = true
			) { m_FromViewable = FromViewable; }

		void SetName (
			const MIUNICODE *name
			) { m_name = name; }

		void SetShowIcon (
			bool ShowIcon
			) { m_ShowIcon = ShowIcon; }

	protected:

		//! Get serialization item definition table.
		static const SERIALIZERITEM* SerialGetItemDef (
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		RVC::OBJITEM m_ObjItemScript;
		MISTRING m_scriptstr;
		FIXEDSTRING<64> m_name;
		char m_IconGroup[16];
		char m_IconName[16];
		mutable MICONSTRUCT* m_icon;
		bool m_ShowIcon;
		bool m_FromViewable;
		bool m_bIsEncrypted;

		void Copy (const VIEWSCRIPT& rhs);
		ERRVALUE LoadLow ();

		virtual bool v_EditProperties (
			MDLGPARENT dlgparent,
			GRE::VIEWABLE *viewable,
			GRE::VIEW *view,
			bool CanChangeSet
			) = 0;

		#endif // GENERATING_DOXYGEN_OUTPUT
	};


//===================================================================================================================

class VIEWSCRIPT::CONTEXT {
	public:

		SMLCONTEXT* GetSMLContext (
			) const { return (m_smlcontext); }

		ERRVALUE ParseScript (
			const VIEWSCRIPT& script
			);

	protected:

		CONTEXT (
			GRE::VIEWABLE *viewable,
			GRE::VIEW *view
			);

		~CONTEXT ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		SMLCONTEXT *m_smlcontext;

		CONTEXT (const CONTEXT&);
		CONTEXT& operator= (const CONTEXT&);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};


//===================================================================================================================

class MACROSCRIPT : public VIEWSCRIPT {
	public:

		enum BUTTONTYPE {
			BUTTONTYPE_Simple =	0,
			BUTTONTYPE_Menu =		1
			};

		class CONTEXT;

		//! Default constructor.
		MACROSCRIPT (
			): m_ButtonType(BUTTONTYPE_Simple) { }

		//! Copy constructor.
		MACROSCRIPT (
			const MACROSCRIPT& rhs
			);

		//! Destructor.
		virtual ~MACROSCRIPT (
			);

		//! Assignment.
		MACROSCRIPT& operator= (
			const MACROSCRIPT& rhs
			);

		BUTTONTYPE GetButtonType (
			) const { return (static_cast<BUTTONTYPE>(m_ButtonType)); }

		const MISTRINGLIST& GetMenuChoiceList (
			) const { return (m_MenuChoiceList); }

		virtual void IniWrite (
			const char* IniGroup,
			const char* IniBaseName
			) const;

		virtual int Load (
			const char* IniGroup = 0,
			const char* IniBaseName = 0
			);

		virtual ERRVALUE SerialRead (
			SERIALIZER& serializer
			);

		virtual ERRVALUE SerialWrite (
			SERIALIZER& serializer,
			const char *tagname = 0
			) const;

		void SetButtonType (
			BUTTONTYPE ButtonType
			) { m_ButtonType = static_cast<UINT8>(ButtonType); }

		void SetMenuChoices (
			const MISTRING& string
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		UINT8 m_ButtonType;
		MISTRINGLIST m_MenuChoiceList;

		static ERRVALUE SerialCB_MenuChoice (SERIALIZER&, const SERIALIZERITEM*, MACROSCRIPT *script, SERIALIZERITEM::ACTION);
		static const SERIALIZERITEM* SerialGetItemDef ();

		virtual bool v_EditProperties (
			MDLGPARENT dlgparent,
			GRE::VIEWABLE *viewable,
			GRE::VIEW *view,
			bool CanChangeSet
			);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};
class MACROSCRIPTLIST : public MILIST<MACROSCRIPT> { };

//! Dialog to edit list of MacroScripts for specified viewable and optional view.
//! @return true if changed, false if not.
bool DlgEditMacroScriptList (
	MDLGPARENT dlgparent,
	MACROSCRIPTLIST& ScriptList,
	GRE::VIEWABLE *viewable,
	GRE::VIEW *view = 0
	);


//===================================================================================================================

class MACROSCRIPT::CONTEXT : public VIEWSCRIPT::CONTEXT {
	public:

		//! Constructor.
		CONTEXT (
			GRE::VIEWABLE *viewable,
			GRE::VIEW *view,
			bool AddAllSymbols = false
			);

		//! Destructor.
		virtual ~CONTEXT ();

		//! Create context for specified script.
		ERRVALUE Create (
			const MACROSCRIPT& script
			);

		//! Get script that was specified when context created.
		const MACROSCRIPT& GetScript (
			) const { return (m_script); }

		void OnCmdRun (MGUI::ID, bool);
		void OnCmdRunMenu (MGUI::ID, bool);

		//! Run script.
		ERRVALUE Run (
			);

		//! Run script with specified menu choice.
		ERRVALUE Run (
			const MIUNICODE* menuchoice
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		MACROSCRIPT m_script;
		SMLSYMBOL *m_MenuChoiceSym;

		CONTEXT (const CONTEXT&);
		CONTEXT& operator= (const CONTEXT&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================

class TOOLSCRIPT : public VIEWSCRIPT {
	public:

		class CONTEXT;

		TOOLSCRIPT () { }

		TOOLSCRIPT (
			const TOOLSCRIPT& rhs
			): VIEWSCRIPT(rhs) { }

		virtual ~TOOLSCRIPT (
			) { }

		TOOLSCRIPT& operator= (
			const TOOLSCRIPT& rhs
			) {
			if (this != &rhs) {
				static_cast<VIEWSCRIPT*>(this)->operator=(rhs);
				}
			return (*this);
			}

		virtual ERRVALUE SerialRead (
			SERIALIZER& serializer
			);

		virtual ERRVALUE SerialWrite (
			SERIALIZER& serializer,
			const char *tagname = 0
			) const;

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		virtual bool v_EditProperties (
			MDLGPARENT dlgparent,
			GRE::VIEWABLE *viewable,
			GRE::VIEW *view,
			bool CanChangeSet
			);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};
class TOOLSCRIPTLIST : public MILIST<TOOLSCRIPT> { };

//! Dialog to edit list of ToolScripts for specified viewable and optional view.
//! @return true if changed, false if not.
bool DlgEditToolScriptList (
	MDLGPARENT dlgparent,
	TOOLSCRIPTLIST& ScriptList,
	GRE::VIEWABLE *viewable,
	GRE::VIEW *view = 0
	);


//===================================================================================================================

//! Context for editing or running ToolScript.
class TOOLSCRIPT::CONTEXT : public VIEWSCRIPT::CONTEXT {
	public:

		//! Constructor.
		CONTEXT (
			GRE::VIEWABLE *viewable,
			GRE::VIEW *view,
			MGUI::TOOL_BASE *pTool = 0
			);

		//! Descturctor.
		virtual ~CONTEXT ();

		//! Call function in script.
		ERRVALUE CallFunction (
			const char *funcname
			) const;

		//! Set symbol indicating whether tool is active.
		void SetActive (
			bool IsActive
			);

		//! Set pointer position and button/modifier state values
		void SetPointerButtonState (
			int x,
			int y,
			MGUI::KEYSTATE keystate,
			MGUI::POINTERBUTTON button
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		SMLSYMBOL *m_SymToolIsActive;
		SMLSYMBOL *m_SymPointerX;
		SMLSYMBOL *m_SymPointerY;
		SMLSYMBOL *m_SymShiftPressed;
		SMLSYMBOL *m_SymCtrlPressed;
		SMLSYMBOL *m_SymLeftButtonPressed;
		SMLSYMBOL *m_SymRightButtonPressed;
		SMLSYMBOL *m_SymMiddleButtonPressed;
		SML::GUI_GADGET *m_pGadget;

		// Unimplemented
		CONTEXT (const CONTEXT&);
		CONTEXT& operator= (const CONTEXT&);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================

}	// End namespace GRE


#endif	// INC_GRE_VIEWSCRIPT_H && USE_MGUI
