/**
 * \file mgui/dlgfusiontables.h
 *
 * \if NODOC
 * $Id: dlgfusiontables.h_v 1.6 2013/01/22 14:51:44 mju Exp $
 *
 * $Log: dlgfusiontables.h_v $
 * Revision 1.6  2013/01/22 14:51:44  mju
 * Remove unused do-nothing getAuthHeader fn.
 *
 * Revision 1.5  2012/12/04 23:07:24  bcolombe
 * Add google api key
 *
 * Revision 1.4  2012/10/02 19:35:25  ldyas
 * changed to gettableauth is public and useful outside of dlg.
 *
 * Revision 1.3  2012/09/11 16:11:40  bcolombe
 * Changes to support in object selection dlg
 *
 * Revision 1.2  2012/08/09 15:06:03  ldyas
 * added GetAuthHeader method
 *
 * Revision 1.1  2012/06/06 22:01:40  ldyas
 * Initial revision
 * \endif
****************************************************************************/

#ifndef	INC_MGUI_DLGFUSIONTABLES_H
#define	INC_MGUI_DLGFUSIONTABLES_H

#ifndef	INC_MGUI_DLGSHELL_H
	#include <mgui/dlgshell.h>
#endif

#ifndef INC_MGUI_EDIT_H
	#include <mgui/edit.h>
#endif

#ifndef INC_MI32_XML_H
	#include <mi32/xml.h>
#endif

namespace MGUI {

class DLG_FUSIONTABLES : public MGUI::DLGSHELL {
	public:
		typedef fastdelegate::FastDelegate<void(XMLNODE*)> DELEGATE_ONGETFUSIONTABLES;

		DLG_FUSIONTABLES (XMLNODE* node);

		~DLG_FUSIONTABLES ();

		//! Generate My Maps list
		void GetFusionTables (const MISTRING& user, const MISTRING& password, bool notify = false);

		//! get data
		void GetTableCSV (const MISTRING& docid,MISTRING& csv);

		//! Set delegate to call when mymaps have been successfully retrieved
		//! Passes node that was created containing mymaps items
		void SetDelegateOnGetFusionTables (
			DELEGATE_ONGETFUSIONTABLES delegate
			) { m_DelegateOnGetFusionTables = delegate; }

		//! Set node to create mymaps items under
		void SetRootNode (XMLNODE* node) { m_rootnode = node; }

		ERRVALUE GetGoogleAuth (const MISTRING& user, const MISTRING& password, MISTRING& returncode);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		XMLNODE *m_rootnode;
		MISTRING m_AuthString;
		DELEGATE_ONGETFUSIONTABLES m_DelegateOnGetFusionTables;

		MGUI::CTRL_EDIT_STRING m_UserName;
		MGUI::CTRL_EDIT_STRING m_Password;
		MGUI::CTRL_TOGGLEBUTTON m_Remember;
		MGUI::CTRL_TEXT m_Content;
		MGUI::CTRL_PUSHBUTTON m_BtnGoogleKey;
		MGUI::CTRL_EDIT_STRING m_FormGoogleKey;

		void EnableGetFusionTablesBtn();
		void DisableGetFusionTablesBtn();
		void AddUserNode(XMLNODE*& usernode, const MISTRING& user);
		void OnGetGoogleKey ();
		void OnEditKey ();

		virtual ERRVALUE v_CreateContent ();
		virtual void v_OnOK ();
		#endif
	};

} // End namespace MGUI

#endif // INC_MGUI_DLGFUSIONTABLES_H
