/**
 * \file mgui/formnrgl.h 
 *	\brief FORM_NAMEDREGION_LIST class definitions.
 *
 * \if NODOC
 * $Id: formnrgl.h_v 1.10 2011/11/29 22:26:08 mju Exp $
 *
 * $Log: formnrgl.h_v $
 * Revision 1.10  2011/11/29 22:26:08  mju
 * Remove redundant hdr.
 *
 * Revision 1.9  2008/09/25 20:26:07  mju
 * Use ctrl_grid instead of ctrl_grid_t.
 *
 * Revision 1.6  2004/05/25 15:09:36  mju
 * OnEndLabelEdit now uses non-const string.
 *
 * Revision 1.1  2001/01/24 20:25:25  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MGUI_FORMNRGL_H
#define	INC_MGUI_FORMNRGL_H

#ifndef	INC_MGUI_GRID_H
#include <mgui/grid.h>
#endif

#ifndef	INC_MI32_NAMEDRGN_H
#include <mi32/namedrgn.h>
#endif

namespace MGUI {

class FORM_NAMEDREGION_LIST : public FORM_COMPOSITE {
	public:

		//! Constructor.
		FORM_NAMEDREGION_LIST (
			NAMEDREGION_LIST& RegionList
			);

		//! Destructor.
		virtual ~FORM_NAMEDREGION_LIST (
			);

		//! Create the form.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,
			MGUI::LAYOUT_SIZEALIGN sizing = MGUI::LAYOUT_SIZEALIGN_Expand
			);

		//! Get reference to currently selected region.
		//! Make sure a region is selected using IsRegionSelected().
		const NAMEDREGION& GetSelectedRegion (
			) const { return (GetSelRegion()); }

		//! Determine if a region is currently selected in the list.
		bool IsRegionSelected (
			) const { return (m_ListCtrl.GetSelectedCount() > 0); }

		//! Set delegate to call when selection changes.
		void SetDelegateOnChangeSelection (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnChangeSelection = delegate; }

	protected:

		//! Called when a region gets selected or all regions become unselected.
		//! Default implementation invokes corresponding delegate.
		//! If override call when/if needed to invoke delegate.
		virtual void v_OnChangeSelection ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		class LISTOBSERVER : public NAMEDREGION_LIST::OBSERVER {
			public:
				LISTOBSERVER (
					FORM_NAMEDREGION_LIST& form
					);
				virtual void OnChanged (const NAMEDREGION& item);
				virtual void OnClear ();
				virtual void OnInsert (const NAMEDREGION& item);
				virtual void OnRemove (const NAMEDREGION& item);
			private:
				FORM_NAMEDREGION_LIST& m_form;
			};

		DELEGATE_VOID_NOPARMS m_DelegateOnChangeSelection;
		NAMEDREGION_LIST& m_RegionList;
		LISTOBSERVER m_ListObserver;
		MGUI::CTRL_GRID m_ListCtrl;
		MGUI::CTRL_PUSHBUTTON m_AddPB;
		MGUI::CTRL_PUSHBUTTON m_SaveAsPB;
		MGUI::CTRL_PUSHBUTTON m_RemovePB;
		MGUI::CTRL_PUSHBUTTON m_CombinePB;
		MDLGPARENT m_DlgParent;

		void AddListCtrlItem (const NAMEDREGION&);
		NAMEDREGION& GetSelRegion () const;
		void OnAdd ();
		void OnCombine ();
		bool OnListCtrlBeginLabelEdit (int, int);
		void OnListCtrlItemSelect (int itemnum, int subitem);
		bool OnListCtrlEndLabelEdit (int, int, MISTRING&);
		void OnRemove ();
		void OnSaveAs ();

		// UNIMPLEMENTED / UNSUPPORTED
		FORM_NAMEDREGION_LIST (const FORM_NAMEDREGION_LIST&);
		FORM_NAMEDREGION_LIST& operator= (const FORM_NAMEDREGION_LIST&);

		friend class LISTOBSERVER;
		friend class LISTCTRL;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


}	// End namespace MGUI

#endif	// INC_MGUI_FORMNRGL_H
