/**
 * \file mgui/toolselectpoint.h
 * \brief MGUI::TOOL_SELECTPOINT class definition
 *
 * \if NODOC
 * $Id: toolselectpoint.h_v 1.9 2011/12/09 20:23:46 mju Exp $
 *
 * $Log: toolselectpoint.h_v $
 * Revision 1.9  2011/12/09 20:23:46  mju
 * No context in v_draw.
 *
 * Revision 1.8  2008/04/16 13:41:43  mju
 * Rename method as setCursors.
 *
 * Revision 1.7  2008/02/01 15:01:54  mju
 * Change button handlers to return bool.
 *
 * Revision 1.6  2008/01/04 19:00:00  mju
 * Generalize pointer button methods.
 *
 * Revision 1.5  2007/12/10 16:17:34  mju
 * Add cursor control, including via delegate with optional delay.
 * Add separate methods to get current point vs point where button pressed.
 *
 * Revision 1.4  2007/10/05 21:12:35  mju
 * Rename inclusion guards.
 *
 * Revision 1.3  2003/10/03 19:58:19  linux32build!build
 * Doxygen
 *
 * Revision 1.2  2003/09/15 13:49:32  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.1  2003/01/10 23:09:28  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_TOOLSELECTPOINT_H
#define	INC_MGUI_TOOLSELECTPOINT_H

#ifndef	INC_MGUI_TOOLBASE_H
#include <mgui/toolbase.h>
#endif

#ifndef	INC_MGUI_TIMER_H
#include <mgui/timer.h>
#endif

namespace MGUI {


//! Simple point selection tool.
//!
class TOOL_SELECTPOINT : public TOOL_LIMITED {
	public:

		typedef fastdelegate::FastDelegate<MGUI::CURSORID(const DPOINT2D& pt, bool ButtonDown)> DELEGATE_ONGETCURSOR;

		//! Constructor.
		explicit TOOL_SELECTPOINT (
			TOOLCANVAS *canvas
			);

		//! Destructor.
		virtual ~TOOL_SELECTPOINT ();

		//! Get current tool point.
		//! Only meaningful while handing change notification.
		const DPOINT2D& GetPointCurrent (
			) const { return (m_PointCurrent); }

		//! Get tool point where button was pressed.
		//! Only meaningful while handing change notification.
		const DPOINT2D& GetPointPressed (
			) const { return (m_PointPressed); }

		//! Set cursors to use for tool while in "valid area".
		//! If not specified, the "thin crosshair" cursor will be used.
		void SetCursors (
			CURSORID CursorIdNormal,				//!< Normal (button-up) cursor
			CURSORID CursorIdDown					//!< Id of cursor while button is down
			) { m_CursorIdNormal = CursorIdNormal; m_CursorIdDown = CursorIdDown; }

		//! Set delegate to call to determine cursor.
		void SetDelegateOnGetCursor (
			DELEGATE_ONGETCURSOR delegate,		//!< Delegate to call
			double delay = 0							//!< Time (in seconds) pointer must be stationary before delegate is called
			) { m_DelegateOnGetCursor = delegate; m_GetCursorDelay = delay; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		CURSORID m_CursorIdNormal;
		CURSORID m_CursorIdDown;
		DELEGATE_ONGETCURSOR m_DelegateOnGetCursor;
		double m_GetCursorDelay;
		DPOINT2D m_PointCurrent;				// Current tool position
		DPOINT2D m_PointPressed;				// Position where button was pressed
		bool m_IsInsideLimit;					// Inside limit of tool
		MGUI::TIMER m_Timer;

		void ComputePointerCursor (const DPOINT2D&, bool ButtonDown = false);
		void OnTimer ();

		// TOOL_BASE overrides.
		virtual void v_DoDraw ();
		virtual bool v_OnPointerButtonDown (MGUI::POINT point, POINTERBUTTON button, KEYSTATE keystate);
		virtual bool v_OnPointerButtonUp (MGUI::POINT point, POINTERBUTTON button, KEYSTATE keystate);
		virtual void v_OnPointerMove (MGUI::POINT point, POINTERBUTTON button, KEYSTATE keystate);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


}	// End namespace MGUI


#endif	// INC_MGUI_TOOLSELECTPOINT_H
