/**
 * \file <mi32/colorspreadmgr.h>
 * \brief COLORSPREADMGR definitions.
 *
 * \if NODOC
 * $Id: colorspreadmgr.h_v 1.3 2010/09/02 17:37:10 mju Exp $
 *
 * $Log: colorspreadmgr.h_v $
 * Revision 1.3  2010/09/02 17:37:10  mju
 * Use colorspread class.
 *
 * Revision 1.2  2010/06/14 14:17:12  mju
 * Add type enums.
 *
 * Revision 1.1  2010/06/11 16:55:13  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_COLORSPREADMGR_H
#define	INC_MI32_COLORSPREADMGR_H

#ifndef	INC_MI32_COLORSPREAD_H
#include <mi32/colorspread.h>
#endif


#ifndef	GENERATING_DOXYGEN_OUTPUT
class MISTRINGLIST;
#endif


//! Manager for color spreads.
class COLORSPREADMGR {
	public:

		//! Compute color spread.
		static double Compute (
			COLOR_ARRAY& ColorArray,	//!< Array to contain 
			const COLOR& ColorBegin,	//!< Beginning color
			const COLOR& ColorEnd,		//!< Ending color
			COLORSPREAD::PATH path,		//!< Spread path
			int NumColors,					//!< Number of colors to compute (or replace if StartIndex != -1)
			int StartIndex = -1			//!< Starting index in existing array to replace, -1 to replace all
			);

		//! Get list of standard color spreads.
		static const MISTRINGLIST& GetNameListStd ();

		//! Get list of user-saved color spreads.
		static const MISTRINGLIST& GetNameListUser ();

		//! Get title of standard color spread to display.
		static MISTRING GetTitleStd (
			const MISTRING& name				//!< Standard color spread name
			);

		//! Load standard color spread.
		static ERRVALUE LoadStd (
			const MISTRING& name,
			COLOR_ARRAY& ColorArray,
			int NumColors = 0					//!< Number of colors in resulting array, 0 to use as originally defined
			);

		//! Load user-saved color spread.
		static ERRVALUE LoadUser (
			const MISTRING& name,
			COLOR_ARRAY& ColorArray,
			int NumColors = 0					//!< Number of colors in resulting array, 0 to use as originally defined
			);

		//! Save user-defined color spread.
		static ERRVALUE SaveUser (
			const MISTRING& name,
			const COLOR_ARRAY& ColorArray
			);

	private:
		#ifndef	GENERATING_DOXYGEN_OUTPUT
		#endif
	};

#endif	// INC_MI32_COLORSPREADMGR_H

