/**
 * \file crypto.h <mi32/crypto.h>
 * \brief Definitions for MCRYTO structure
 *
 * \if NODOC
 * $Id: crypto.h_v 1.6 2006/07/10 19:10:44 dwilliss Exp $
 *
 * $Log: crypto.h_v $
 * Revision 1.6  2006/07/10 19:10:44  dwilliss
 * added a flag
 *
 * Revision 1.5  2005/03/31 16:57:10  fileserver!dwilliss
 * Rename one of our types to MIUNICODE because it conflicted with a Microsoft #define
 *
 * Revision 1.4  2005/01/07 15:22:13  mju
 * Move Mcrypto prototypes here.
 *
 * Revision 1.3  2003/09/15 13:49:56  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.2  1999/04/09 17:33:48  mju
 * Change ending comment.
 *
 * Revision 1.1  1999/04/08  14:33:26  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_CRYPTO_H
#define	INC_MI32_CRYPTO_H

#ifdef RVCSYSDLL
	#define LIBEXPORT MI_DLLEXPORT
#else
	#define LIBEXPORT MI_DLLIMPORT
#endif

/*** !!! This structure cannot be changed without significant alterations to rvclow.dll !!! */

struct MCRYPTOHDR {
	UINT32 flags;		//!<  See below 
	int keynumber;		//!<  For prompting only.  Not used to decrypt 
	int method;			//!<  Encryption method for future expansion 
	};

#define MCRYPTO_NeedPassword			0x00000001
#define MCRYPTO_NeedAnyValidKey		0x00000002
#define MCRYPTO_NeedSpecificKey		0x00000004
#define MCRYPTO_ASCII					0x00000008	//!<  uuencode into ASCII range for text file 
#define MCRYPTO_UsableByLite			0x00000010	//!< Can be used by TNTlite (need special license option to encrypt this way)


//!  Read the crypto header.  
//!
//! @return 1 if encrypted, 0 if not.  
//!
//! Note that the header is only used as an aid to tell us what decryption method to use.  You could 
//! hack the header to make it not ask for a password, but it won't decrypt without it. 
LIBEXPORT int McryptoReadHeader (
	MIUNICODE *ucbuf,							//!< Unicode buffer returned
	MCRYPTOHDR *hdr,							//!< Crypto header returned
	UINT32 flags								//!< Flags
	);

//! Encode the input.
LIBEXPORT int McryptoEncode (
	MIUNICODE **outbuf,							//!< Output buffer
	MIUNICODE *inbuf,							//!< Input buffer
	MIUNICODE *password,						//!< Password
	int keynum,									//!< Key number
	UINT32 flags								//!< Flags
	);

//! If not encoded, just copies the input.
LIBEXPORT int McryptoDecode (
	MIUNICODE **outbuf,							//!< Output buffer
	MIUNICODE *inbuf,							//!< Input buffer
	MIUNICODE *password,						//!< Password
	UINT32 flags								//!< Flags
	);

#undef	LIBEXPORT

#endif	// INC_MI32_CRYPTO_H 
