/**
 * \file filelegacy.h <mi32/filelegacy.h>
 * \brief Legacy file stream/handle support for interface to 3rd-party libraries
 *
 * \if NODOC
 * $Id: filelegacy.h_v 1.2 2005/07/20 14:25:28 mju Exp $
 *
 * $Log: filelegacy.h_v $
 * Revision 1.2  2005/07/20 14:25:28  mju
 * Include stdio hdr for platform-specific FILE typedef.
 *
 * Revision 1.1  2005/07/20 14:18:42  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_FILELEGACY_H
#define	INC_MI32_FILELEGACY_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

#ifndef	INC_MI32_FILEACCESS_H
#include <mi32/fileaccess.h>
#endif

#ifndef	INC_STDIO_H
#include <stdio.h>
#define	INC_STDIO_H
#endif

#ifdef MISYSTEMDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class FILEPATH;
#endif


//! Class to provide open/close access as "handle" (int) for 3rd-party libraries.
//! Provides implicit cast to int handle.
class CLASSLIBEXPORT FILE_LEGACY_HANDLE {
	public:

		//! Default constructor
		FILE_LEGACY_HANDLE (
			);

		//! Destructor, closes file if open.
		~FILE_LEGACY_HANDLE (
			);

		//! Cast to legacy "handle".
		operator int (
			) const { return (m_handle); }

		//! Close file if open.
		void Close (
			);

		//! Get legacy handle.
		int GetHandle (
			) const { return (m_handle); }

		//! Determine if file is currently open.
		bool IsOpen (
			) const { return (m_handle >= 0); }

		//! Open or create specified file.
		//! If file is already open the previous file will be closed.
		//! Note that sharing modes are not supported on all platforms and filesystems.  If specified
		//! sharing mode is not supported then will open with SHARE_Any.
		ERRVALUE Open (
			const FILEPATH& filepath,					//!< Path to file to be opened or created
			FILE_ACCESS access = FILE_ACCESS_Read,	//!< Desired access
			FILE_SHARE share = FILE_SHARE_Auto		//!< Sharing mode
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		int m_handle;

		FILE_LEGACY_HANDLE (const FILE_LEGACY_HANDLE&);
		FILE_LEGACY_HANDLE& operator= (const FILE_LEGACY_HANDLE&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


//! Class to provide open/close access as "file pointer" (FILE*) for 3rd-party libraries.
//! Provides implicit cast to FILE*.
class CLASSLIBEXPORT FILE_LEGACY_STREAM {
	public:

		enum TRANSLATION {
			TRANSLATION_None =	0,		//!< No translation (raw binary
			TRANSLATION_Text =	1		//!< Text translation with platform-specific EOL and EOF support, use with caution
			};

		//! Default constructor
		FILE_LEGACY_STREAM (
			);

		//! Destructor, closes file if open.
		~FILE_LEGACY_STREAM (
			);

		//! Cast to legacy "handle".
		operator FILE* (
			) const { return (m_fp); }

		//! Close file if open.
		void Close (
			);

		//! Get legacy file pointer.
		FILE* GetFilePointer (
			) const { return (m_fp); }

		//! Determine if file is currently open.
		bool IsOpen (
			) const { return (m_fp != 0); }

		//! Open or create specified file.
		//! If file is already open the previous file will be closed.
		ERRVALUE Open (
			const FILEPATH& filepath,					//!< Path to file to be opened or created
			TRANSLATION translation,					//!< Translation mode
			FILE_ACCESS access = FILE_ACCESS_Read	//!< Desired access
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		FILE *m_fp;

		FILE_LEGACY_STREAM (const FILE_LEGACY_STREAM&);
		FILE_LEGACY_STREAM& operator= (const FILE_LEGACY_STREAM&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


#undef	CLASSLIBEXPORT

#endif	// INC_MI32_FILELEGACY_H
