/**
 * \file mi32/lidarlasscan.h
 * \brief LIDAR::LAS SCANNER definitions
 *
 * \if NODOC
 * $Id: lidarlasscan.h_v 1.5 2012/09/14 17:25:08 mju Exp $
 *
 * $Log: lidarlasscan.h_v $
 * Revision 1.5  2012/09/14 17:25:08  mju
 * Add isNeeded.
 *
 * Revision 1.4  2012/09/11 20:03:29  mju
 * Remove scanvalues, now creates vlrs in las/lasx file.
 *
 * Revision 1.3  2012/08/24 17:18:56  mju
 * Change scanner to not be based on mithread::job.
 *
 * Revision 1.1  2012/07/16 20:41:09  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_LIDARLASSCAN_H
#define	INC_MI32_LIDARLASSCAN_H

#ifndef	INC_MI32_FILEPATH_H
#include <mi32/filepath.h>
#endif

#ifdef RVCSYSDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif

namespace LIDAR {
namespace LAS {

//=====================================================================================================================

//! Job to perform background file scan.
//! All scan records will be added to file on completion.
class CLASSLIBEXPORT SCANNER {
	public:

		//! Construct for specified FILEPATH.
		explicit SCANNER (
			const FILEPATH& filepath
			);

		//! Destructor.
		~SCANNER ();

		//! Determine if scan is needed.
		//! @return TRUE if needed, FALSE if not, < 0 if error
		int IsNeeded () const;

		//! Run scanner.
		ERRVALUE Run ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		FILEPATH m_filepath;
		#endif
	};

//=====================================================================================================================

}	// End namespace LAS
}	// End namespace LIDAR

#undef	CLASSLIBEXPORT

#endif	// INC_MI32_LIDARLASSCAN_H
