/**
 * \file mxtools.h <mi32/mxtools.h>
 * \brief Definitions for MicroImages library tool functions
 *
 * \if NODOC
 * $Id: mxtools.h_v 1.112 2011/11/04 12:51:57 mju Exp $
 *
 * $Log: mxtools.h_v $
 * Revision 1.112  2011/11/04 12:51:57  mju
 * Remove redundant hdr.
 *
 * Revision 1.111  2008/11/20 18:12:05  mju
 * Use uppercase mat3x3.
 *
 * Revision 1.110  2008/11/19 22:38:20  mju
 * Use trans2d_mapgen ptr for viewToTool trans.
 *
 * Revision 1.109  2008/11/19 18:20:38  mju
 * Use trans2d_mapgen in setOutputTrans.
 *
 * Revision 1.107  2008/02/18 13:45:13  mju
 * Add mxtCR_Wheel...
 *
 * Revision 1.106  2007/08/21 14:56:16  scowan
 * Made string param const.
 *
 * Revision 1.105  2005/06/03 14:36:21  mju
 * Add getTrans2D and setTrans2D variant using trans2d_affine.
 *
 * Revision 1.104  2004/06/30 21:17:56  scowan
 * Changed view set function name.
 *
 * Revision 1.102  2004/06/29 21:20:00  scowan
 * Changed region tool to use region 2d.
 *
 * Revision 1.101  2004/06/28 23:48:18  scowan
 * Changed to use region 2d.
 *
 * Revision 1.90  2002/02/07 00:06:05  scowan
 * Changed leader line information.
 *
 * Revision 1.89  2002/01/17 18:53:34  scowan
 * Added text label tool info.
 *
 * Revision 1.87  2001/10/23 15:21:50  scowan
 * Added to polylie editor.
 *
 * Revision 1.85  2000/06/14 21:10:24  mju
 * Change MxtCreate... macros to inlines so will be documented by Genitor.
 *
 * \endif
**/

#ifndef	INC_MI32_MXTOOLS_H
#define	INC_MI32_MXTOOLS_H

#ifndef	INC_MI32_XDEFNS_H
	#include <mi32/xdefns.h>
#endif

#ifndef	INC_MI32_UNITCONV_H
	#include <mi32/unitconv.h>
#endif

#ifndef	INC_MI32_TRANS2DC_H
	#include <mi32/trans2dc.h>
#endif

#ifndef	INC_MI32_REGION2D_H
	#include <mi32/region2d.h>
#endif

#ifndef	INC_MI32_ELEMSTYL_H
	#include <mi32/elemstyl.h>
#endif

#ifndef	INC_MI32_MCB_H
	#include <mi32/mcb.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
namespace SPATREF {
	class COORDREFSYS;
	}
class TRANS2D_AFFINE;
#endif // GENERATING_DOXYGEN_OUTPUT

typedef void *MXTHANDLE;

/*
* Alter tool position flags
* for MxtMove(), MxtResize(), MxtShear(), MxtRotate().
*/

#define	MxtADJUST_Relative		0x0000
#define	MxtADJUST_Absolute		0x0001
#define	MxtADJUST_ToolCoords		0x0000
#define	MxtADJUST_ScreenCoords	0x0002
#define	MxtADJUST_Notify			0x8000


/********	Change reason flags 	**********/

#define MxtCR_ReasonMask	0xFF000000
#define MxtCR_IsBegin		0x10000000
#define MxtCR_IsContinue	0x20000000
#define MxtCR_IsEnd			0x40000000
#define MxtCR_IsOther		0x80000000
#define MxtCR_IsActivate	0x01000000
#define MxtCR_IsDestroy		0x02000000

#define MxtCR_NoReason				(0|MxtCR_IsOther)		/* "No reason, I just like doing things like that." */
#define MxtCR_Activate				(1|MxtCR_IsOther|MxtCR_IsActivate)
#define MxtCR_SetValues				(2|MxtCR_IsOther)
#define MxtCR_NewLimits				(3|MxtCR_IsOther)
#define MxtCR_SelectPosition		(4|MxtCR_IsOther)
#define MxtCR_Manage					(5|MxtCR_IsOther)
#define MxtCR_Unmanage				(6|MxtCR_IsOther)
#define MxtCR_ClearPosition		(7|MxtCR_IsOther)
#define MxtCR_Cancel					(8|MxtCR_IsOther)		/* User pressed 'Cancel' button */
#define MxtCR_Destroy				(10|MxtCR_IsDestroy)
#define MxtCR_ChangeCurrentElem	(11|MxtCR_IsOther)

#define MxtCR_MoveBegin				(30|MxtCR_IsBegin)
#define MxtCR_MoveContinue			(30|MxtCR_IsContinue)
#define MxtCR_MoveEnd				(30|MxtCR_IsEnd)
#define MxtCR_ResizeBegin			(31|MxtCR_IsBegin)
#define MxtCR_ResizeContinue		(31|MxtCR_IsContinue)
#define MxtCR_ResizeEnd				(31|MxtCR_IsEnd)
#define MxtCR_RotateBegin			(32|MxtCR_IsBegin)
#define MxtCR_RotateContinue		(32|MxtCR_IsContinue)
#define MxtCR_RotateEnd				(32|MxtCR_IsEnd)
#define MxtCR_ShearBegin			(33|MxtCR_IsBegin)
#define MxtCR_ShearContinue		(33|MxtCR_IsContinue)
#define MxtCR_ShearEnd				(33|MxtCR_IsEnd)
#define MxtCR_EditBegin				(34|MxtCR_IsBegin)
#define MxtCR_EditContinue			(34|MxtCR_IsContinue)
#define MxtCR_EditEnd				(34|MxtCR_IsEnd)
#define MxtCR_CreateBegin			(35|MxtCR_IsBegin)
#define MxtCR_CreateContinue		(35|MxtCR_IsContinue)
#define MxtCR_CreateEnd				(35|MxtCR_IsEnd)
#define MxtCR_Trace					(36|MxtCR_IsOther)
#define MxtCR_WheelForward			(37|MxtCR_IsOther)
#define MxtCR_WheelBackward		(38|MxtCR_IsOther)

#define MxtCR_CreateTool			MxtCR_CreateBegin		//!<  Compatibility to old stuff 

struct SNAPGRIDPARMS {
	DPOINT3D BasePt;							//!<  Base point of snap grid 
	double xsize;								//!<  X direction size 
	double ysize;								//!<  Y direction size 
	double zsize;								//!<  Z direction size 
	double rotate;								//!<  Rotation angle (snap grid) 
	MAT3X3 OtoS, StoO;						//!<  Translation to/from snap grid coordinates 
	};

struct MANUALTOOLPARMS {
	TRANS2D_MAPGEN *m_pTransViewToTool;	//!< Transformation from view to tool coordinates, set output side only
	UNITCONV units;							//!<  Original tool coordinate units 
	double xscaletou;							//!<  Scale to units defined in UNITCONV 
	double yscaletou;
	double zscaletou;
	unsigned scalevaluesset:1,				//!<  Scale values are set 
				unitconvset:1,					//!<  Unit conversion fields are set 
				mapparmset:1,					//!<  A valid map projection is in use for view 
				unused:29;
	MANUALTOOLPARMS ()
		: m_pTransViewToTool(0) {}
	};


struct MxtAnyCallbackStruct {
	UINT32 reason;								//!<  Reason for callback, defined above 
	MXTHANDLE mxt;								//!<  Tool handle, useful to get user data 
	XAnyEvent *event;							//!<  X event that initiated the callback 
	void *toolinfo;							//!<  Pointer to the internal tool data, RO 
	unsigned IsGPS:1,
				IsXYDig:1,
				unused:30;
	};


/* Flags common to ALL tools, allocated MSB first */

#define	MxtFLAG_DialogManipMask	0xF8000200		//!<  Mask for flags that manipulate dialog stuff 
#define	MxtFLAG_EnableXYDig		0x10000000		//!<  Places a X-Y Digitizer toggle button on the dialog 
#define	MxtFLAG_EnableManual		0x20000000		//!<  Allow user to edit tool manually, places manual button on dialog 
#define	MxtFLAG_EnableDialog		0x40000000		//!<  Allow tool to generate a dialog, not optional for some tools 
#define	MxtFLAG_EnableSnapGrid	0x80000000		//!<  Places a Snap Grid toggle button on the dialog 

#define	MxtFLAG_EnableGPS			0x08000000		//!<  Enable GPS source to be used 
#define	MxtFLAG_NoManualProj		0x04000000		//!<  DO not allow manual projection / distance to be shown on manual dialog 

#define	MxtFLAG_NoResize			0x00100000
#define	MxtFLAG_NoCreate			0x00200000
#define	MxtFLAG_CanMoveOff		0x00400000		//!<  Tool graphic can be moved off the screen 
#define	MxtFLAG_NoRotate			0x00800000

#define	MxtFLAG_NotifyMotion		0x00010000		//!<  Send "ChangeCB" events while tool being moved 
#define	MxtFLAG_NoResetDialog	0x00020000		//!<  Do not reset dialog upon a activate callback 
#define	MxtFLAG_NoExposeResize	0x00040000		//!<  Ignore Expose/Resize callbacks 
#define	MxtFLAG_NoSaveB			0x00080000		//!<  Don't show save button 

#define	MxtFLAG_HalfStepDeltas	0x00001000		//!<  Take "half steps" in screen coords and do special drawing 
#define	MxtFLAG_ForceActivate	0x00002000		//!<  Call "Activate" callback even if tool does not have a position set 
#define	MxtFLAG_AllowCancelBtn	0x00004000		//!<  Allow 'Cancel' button in dialog, caller must support MxtCR_Cancel callback 
#define	MxtFLAG_NoDialog			0x00008000		//!<  No dialog is to be made 

#define	MxtFLAG_ForceManual		0x00000800		//!<  Force manual dialog on, no "Manual Entry" button 
#define	MxtFLAG_NoMove				0x00000400		//!<  Do not allow tool to move 
#define	MxtFLAG_NoElemHeight		0x00000200		//!<  Do not allow "Use Element Height" toggle on text style 

/* Flags for MxtSetBounds() */
#define	MxtBOUND_Xmin				0x01000000
#define	MxtBOUND_Ymin				0x02000000
#define	MxtBOUND_Xmax				0x04000000
#define	MxtBOUND_Ymax				0x08000000
#define	MxtBOUND_FullRect			0x0F000000


/*
* Marker types, used in various tools.  Defaults should be set in mxtE...() functions
*/

#define	MxtMARKER_X				0			//!<  Marker types 
#define	MxtMARKER_Plus			1
#define	MxtMARKER_ArrowIn		2
#define	MxtMARKER_ArrowOut	3
#define	MxtMARKER_None			4
#define	MxtMARKER_Circle		5
#define	MxtMARKER_Flat			6
#define	MxtMARKER_Square		7


/*----------------------------------------------------------------------------*/
/*		Definitions for elastic line tool													*/
/*----------------------------------------------------------------------------*/

struct ELINEINFO {
	DPOINT2D start, end;
	UINT16 smarker, emarker;				//!<  Marker types for start and end 
	UINT16 smarksize, emarksize;			//!<  Size of markers 
	};


/*----------------------------------------------------------------------------*/
/*		Definitions for elastic box tool														*/
/*----------------------------------------------------------------------------*/

struct EBOXINFO {
	double xinit, yinit, xlast, ylast;	//!<  Current box position or extents if rotated 
	DPOINT2D corner[4];						//!<  Box corners 
	double wmin, hmin, wmax, hmax;		//!<  Size range 
	double xsize, ysize;						//!<  Current box size 
	double rotangle;							//!<  Rotation angle in radians, angle of vector from corner 0 to 1, CCW from right 
	double xproport, yproport;				//!<  Ratio for proportional box 
	};

#define	MxEBOX_Range			0x00000001		//!<  Range fields (xinit, yinit, xlast, ylast) set 
#define	MxEBOX_AllCorners		0x00000002		//!<  All corner locations set (DEFAULT) 
#define	MxEBOX_Proport			0x00000004

#define	MxEBOX_Corner0			0x00000010		//!<  Corner 0, size and angle set 
#define	MxEBOX_Corner1			0x00000020		//!<  Corner 1, size and angle set 
#define	MxEBOX_Corner2			0x00000040		//!<  Corner 2, size and angle set 
#define	MxEBOX_Corner3			0x00000080		//!<  Corner 3, size and angle set 

#define	MxEBOX_CanFlipHorz	0x00000100		//!<  Allow user to flip box horizontally 
#define	MxEBOX_CanFlipVert	0x00000200		//!<  Allow user to flip box vertically 
#define	MxEBOX_CanFlipBoth	(MxEBOX_CanFlipHorz|MxEBOX_CanFlipVert)

/* Maximum allowed flag is 0x00000200 !!! */

/*----------------------------------------------------------------------------*/
/*		Definitions for elastic polyline tool	 											*/
/*----------------------------------------------------------------------------*/

struct EPOLYLINEINFO {
	DPOLYGON poly;
	double zvalue;								//!<  Z value for VPOINT_3DXY points 
	unsigned Is3D:1,							//!<  The point data is in (x,y,z) coordinates 
				IsPoly:1,						//!<  Currently in "polygon" mode, assumed to be closed, draw dashed line 
				LockStartPt:1,					//!<  Do not allow start point to be moved or added to 
				LockEndPt:1,					//!<  Do not allow end point to be moved or added to 
				KeepModIndex:1,				//!<  Maintain modification array 
				EnableTraceBtn:1,				//!< Enable trace button
				InTraceMode:1,					//!< In trace mode
				unused:25;						//!<  Pad to 32 bits 
	UINT32 CurveType;							//!<  Type of curve fit to use 
	UINT16 smarker, emarker;				//!<  Marker types for start and end 
	UINT16 smarksize, emarksize;			//!<  Size of markers 
	UINT16 smarkcolor, emarkcolor;		//!<  Color of markers 0 for primary, 1 for secondary 
	INT32 *ModIndex;							//!<  Modification array for line vertices 
	};

#define	MxEPOLYLINE_EditPoly					0x00000001	//!<  Edit polygon 
#define	MxEPOLYLINE_EnableProfile			0x00000004	//!<  Enable profile option 
#define	MxEPOLYLINE_EnableTrace				0x00000008	//!< Enable trace button

#define	MxEPOLYLINE_EnableResetB			0x00000010	//!<  Enable tool reset button 
#define	MxEPOLYLINE_NoClearB					0x00000020	//!<  Don't show clear button 
#define	MxEPOLYLINE_NoClosePolyB			0x00000040	//!<  Don't show close poly button 
#define	MxEPOLYLINE_UseCurveFit				0x00000080	//!<  Edit a curve fit 

#define	MxEPOLYLINE_CanChgLinePolyMode	0x00000100	//!<  User can toggle between polygon and line modes 
#define	MxEPOLYLINE_NoEditZValue			0x00000200	//!<  Don't let user edit z value 

/*----------------------------------------------------------------------------*/
/*		Definitions for elastic multi-polyline tool										*/
/*----------------------------------------------------------------------------*/

struct EMULTILINEINFO {
	DPOLYGON *Lines;							//!<  List of lines to edit 
	UINT32 NumLines;							//!<  Number of lines in the tool 
	INT32 ActiveLine;							//!<  The active line in the tool, -1 if current line has no position set 
	double zvalue;								//!<  Z value for VPOINT_3DXY points 
	unsigned Is3D:1,							//!<  The point data is in (x,y,z) coordinates 
				IsPoly:1,						//!<  Currently in "polygon" mode, assumed to be closed, draw dashed line 
				LockStartPt:1,					//!<  Do not allow start point to be moved or added to 
				LockEndPt:1,					//!<  Do not allow end point to be moved or added to 
				SingleLine:1,					//!<  This is single-line data 
				unused:27;						//!<  Pad to 32 bits 
	UINT32 CurveType;							//!<  Type of curve fit to use 
	UINT16 smarker, emarker;				//!<  Marker types for start and end 
	UINT16 smarksize, emarksize;			//!<  Size of markers 
	UINT16 smarkcolor, emarkcolor;		//!<  Color of markers 0 for primary, 1 for secondary 
	};

#define	MxEMULTILINE_EditPoly				0x00000001	//!<  Edit polygon 
#define	MxEMULTILINE_LineIs3D				0x00000002	//!<  Tool thinks line is 3D, until Is3D variable is changed 
#define	MxEMULTILINE_EnableResetB			0x00000004	//!<  Enable tool reset button 
#define	MxEMULTILINE_NoClearB				0x00000008	//!<  Don't show clear button 

#define	MxEMULTILINE_NoClosePolyB			0x00000010	//!<  Don't show close poly button 
#define	MxEMULTILINE_UseCurveFit			0x00000020	//!<  Edit a curve fit 
#define	MxEMULTILINE_CanChgLinePolyMode	0x00000040	//!<  User can toggle between polygon and line modes 
#define	MxEMULTILINE_NoEditZValue			0x00000080	//!<  Don't let user edit z value 

#define	MxEMULTILINE_SingleLineOnly		0x00000100	//!<  Don't allow multi-line editing 

/*----------------------------------------------------------------------------*/
/*		Definitions for elastic point tool	 												*/
/*----------------------------------------------------------------------------*/

struct EPOINTINFO {
	DPOINT3D pt;
	UINT16 marker;								//!<  Marker to use for center 
	UINT16 markersize;						//!<  Size of marker in pixels 
	};

#define	MxEPOINT_NoEditZValue			0x00000001	//!<  Don't let user edit z value 


/*----------------------------------------------------------------------------*/
/*		Definitions for elastic point tool	 												*/
/*----------------------------------------------------------------------------*/

struct EMULTIPOINTINFO {
	DPOINT3D *pt;
	INT32 NumPoints;
	INT32 ActivePoint;						//!<  The active point in the tool, -1 if current line has no position set 
	UINT16 marker;								//!<  Marker to use for center 
	UINT16 markersize;						//!<  Size of marker in pixels 
	};

#define	MxEMULTIPOINT_NoEditZValue			0x00000001	//!<  Don't let user edit z value 
#define	MxEMULTIPOINT_SinglePointOnly		0x00000002	//!<  Don't allow multi-point editing 

/*----------------------------------------------------------------------------*/
/*		Definitions for elastic circle tool													*/
/*----------------------------------------------------------------------------*/

struct ECIRCLEINFO {
	DPOINT2D center;							//!<  Circle center 
	double radius;								//!<  Circle radius 
	double rmin, rmax;						//!<  Radius range 
	UINT16 cmarker;							//!<  Marker to use for center 
	UINT16 cmarkersize;						//!<  Size of marker in pixels 
	double rotangle;							//!<  Circle bar rotation angle in radians 
	};

#define	MxECIRCLE_Rmin				0x00000001	//!<  Flag bits 
#define	MxECIRCLE_Rmax				0x00000002
#define	MxECIRCLE_RotationBar	0x00000004	//!<  Circle has rotation bar 


/*----------------------------------------------------------------------------*/
/*		Definitions for elastic regular polygon tool										*/
/*----------------------------------------------------------------------------*/

struct EREGPOLYINFO {
	DPOINT2D center;							//!<  Circle center 
	double radius;								//!<  Circle radius 
	double rmin, rmax;						//!<  Radius range 
	double rotangle;							//!<  Rotation angle in radians, angle of vector from center point, CCW from right 
	UINT32 NumSegments;						//!<  Number of segments in polygon 
	UINT16 cmarker;							//!<  Marker to use for center 
	UINT16 cmarkersize;						//!<  Size of marker in pixels 
	};

#define	MxEREGPOLY_Rmin				0x00000001	//!<  Flag bits 
#define	MxEREGPOLY_Rmax				0x00000002


/*----------------------------------------------------------------------------*/
/*		Definitions for elastic arc tools													*/
/*----------------------------------------------------------------------------*/

struct EARCINFO {
	DPOINT2D center;							//!<  Arc center 
	double radius;								//!<  Arc radius 
	int startang;								//!<  Start angle in 1/64 degree 
	int sweepang;								//!<  Sweep angle in 1/64 degree 
	double rmin, rmax;						//!<  Radius range 
	UINT16 cmarker;							//!<  Marker to use for center 
	UINT16 cmarkersize;						//!<  Size of marker in pixels 
	unsigned IsChord:1,						//!<  The tool is drawing a chord 
				IsWedge:1,						//!<  The tool is drawing a wedge 
				unused:30;						//!<  Pad to 32 bits 
	};

#define	MxEARC_Rmin				0x00000001		//!<  Flags for what is initially set in EARCINFO structure 
#define	MxEARC_Rmax				0x00000002
#define	MxEARC_EditWedge		0x00000004
#define	MxEARC_EditChord		0x00000008


/*----------------------------------------------------------------------------*/
/*		Definitions for crosshair tool													   */
/*----------------------------------------------------------------------------*/

struct CROSSHAIRINFO {
	DPOINT2D center;							//!<  Center of crosshair 
	double angle;								//!<  Angle of crosshair in radians 
	double hlen, vlen;						//!<  Length of "horizontal" and "vertical" bars 
	DPOINT2D endpt0, endpt180;				//!<  End points of "horizontal" line 
	DPOINT2D endpt90, endpt270;			//!<  End points of "vertical" line 
	};

#define	MxCROSSHAIR_NoBar0		0x00000001	//!<  Do not draw   0 degree bar 
#define	MxCROSSHAIR_NoBar90		0x00000002	//!<  Do not draw  90 degree bar 
#define	MxCROSSHAIR_NoBar180		0x00000004	//!<  Do not draw 180 degree bar 
#define	MxCROSSHAIR_NoBar270		0x00000008	//!<  Do not draw 270 degree bar 
#define	MxCROSSHAIR_SnapInside	0x00000010	//!<  Snap crosshair to cursor in "inside" zone 


/*----------------------------------------------------------------------------*/
/*		Definitions for elastic ellipse tool												*/
/*----------------------------------------------------------------------------*/

struct EELLIPSEINFO {
	DPOINT2D center;							//!<  Ellipse center 
	double angle;
	double xradius;							//!<  X Ellipse radius 
	double yradius;							//!<  Y Ellipse radius 
	double xrmin, xrmax;						//!<  Radius range 
	double yrmin, yrmax;						//!<  Radius range 
	UINT16 cmarker;							//!<  Marker to user for center 
	UINT16 cmarkersize;						//!<  Size of marker in pixels 
	};

#define	MxEELLIPSE_Rmin		0x00000001		//!<  Flags for what is initially set in EELLIPSEINFO structure 
#define	MxEELLIPSE_Rmax		0x00000002


/*----------------------------------------------------------------------------*/
/*		Definitions for elastic text label tool										*/
/*----------------------------------------------------------------------------*/

class ETEXTLABELINFO {
	public:
	
		enum BASECURVE {
			BASECURVE_Straight,
			BASECURVE_Exact,
			BASECURVE_Spline
			};
			
		struct LEADERINFO {
			POLYLINE m_Line;
			DPOINT3D m_TiePoint;
			INT32 m_ElemNum;
			ELEMTYPE m_ElemType;
			
			LEADERINFO () : m_ElemNum(0), m_ElemType(ELEMTYPE_Empty) {}
			};
			
		POLYLINE m_Baseline;						//!< Baseline points
		BASECURVE m_BaseCurve;			  		//!< Baseline curve
		MISTRING m_String;					  	//!< Text string being edited
		double m_Height;						  	//!< Height of text in tool units
		TEXTSTYLE m_Style;					  	//!< Style information for text
		double m_MapScale;					  	//!< Map scale to use based on the scale of the drawing area
		MILIST<LEADERINFO> m_LeaderLines;	//!< List of leader lines
		POLYLINE m_SourceLine;					//!< Source line to follow if the label wants to be moved "along" the line
		
		ETEXTLABELINFO (
			) :
			m_BaseCurve(BASECURVE_Straight),
			m_Height(1.0),
			m_MapScale(1.0)
			{}
			
		ETEXTLABELINFO (
			const ETEXTLABELINFO& rhs
			) :
			m_Baseline(rhs.m_Baseline),
			m_BaseCurve(rhs.m_BaseCurve),
			m_String(rhs.m_String),
			m_Height(rhs.m_Height),
			m_Style(rhs.m_Style),
			m_MapScale(rhs.m_MapScale),
			m_LeaderLines(rhs.m_LeaderLines),
			m_SourceLine(rhs.m_SourceLine)
			{}

		ETEXTLABELINFO& operator= (
			const ETEXTLABELINFO& rhs
			) {
			if (this != &rhs) {
				m_Baseline = rhs.m_Baseline;
				m_BaseCurve = rhs.m_BaseCurve;
				m_String = rhs.m_String;
				m_Height = rhs.m_Height;
				m_Style = rhs.m_Style;
				m_MapScale = rhs.m_MapScale;
				m_LeaderLines = rhs.m_LeaderLines;
				m_SourceLine = rhs.m_SourceLine;
				}
			return (*this);
			}
			
		void Clear (
			) {
			m_String.Clear();
			m_LeaderLines.Clear();
			m_Baseline.Clear();
			return;
			}
			
	};

#define	MxETEXTLABEL_NoLeaderLines			0x00000001		//!< Do not allow editing of leader lines
#define	MxETEXTLABEL_NoAdvanced				0x00000002		//!< Do not allow advanced style button
#define	MxETEXTLABEL_NoBaselineEdit		0x00000004		//!< Do not allow baseline to be edited
#define	MxETEXTLABEL_NoStylePanel			0x00000008		//!< Do not allow the style panel to be created

PREVENT_MEMFUNC(ETEXTLABELINFO);

/*----------------------------------------------------------------------------*/
/*		Definitions for elastic region tool													*/
/*----------------------------------------------------------------------------*/

struct EREGIONINFO {
	REGION2D region;							//!<  Region to draw/move 
	};
PREVENT_MEMFUNC(EREGIONINFO);

#define	MxEREGION_NoMove			0x00000001


//! Macros for defining class methods to add Mxt callbacks

#define	MXT_ADD(_clsname)		inline void MxtCallbackAdd (MXTHANDLE vmxt, void (*f)(MxtAnyCallbackStruct*,_clsname*), UINT32 flags) const { MxtAddCallback(vmxt,reinterpret_cast<McbCallbackFunc>(f),const_cast<_clsname*>(this),(flags)); }
#define	MXT_REMOVE(_clsname)	inline void MxtCallbackRemove (MXTHANDLE vmxt, void (*f)(MxtAnyCallbackStruct*,_clsname*), UINT32 flags) const { MxtRemoveCallback(vmxt,reinterpret_cast<McbCallbackFunc>(f),const_cast<_clsname*>(this),(flags)); }


/**
*	Function Prototypes
**/

//!:Associate with "Mxt Graphic Tool Functions"
//!\addtogroup Mxt Mxt Graphic Tool Functions
//!@{

extern "C" {

//! Create an instance of a rotatable crosshair tool.
MXTHANDLE mxtCrossHair (
	UINT32 flags, 
	void *pwidget
	);

//! Create an instance of an elastic arc tool.
//!
//! 3/4 Done, Need bound checking fixed.
MXTHANDLE mxtEArc (
	UINT32 flags, 
	void *pwidget
	);
	
//! Create an instance of an elastic box tool.				
MXTHANDLE mxtEBox (
	UINT32 flags, 
	void *pwidget
	);

//! Create an instance of an elastic circle tool.
//!
//! 3/4 Done, Need bound checking fixed. 
MXTHANDLE mxtECircle (
	UINT32 flags, 
	void *pwidget
	);

//! Create an instance of an elastic ellipse tool. 
MXTHANDLE mxtEEllipse (
	UINT32 flags, 
	void *pwidget
	);
	
//! Create an instance of an elastic box tool.		
MXTHANDLE mxtELine (
	UINT32 flags, 
	void *pwidget
	);

//! Create an instance of an elastic point tool.
MXTHANDLE mxtEMultiPoint (
	UINT32 flags, 
	void *pwidget
	);

//! Create multi elastic polyline tool.
MXTHANDLE mxtEMultiPolyLine (
	UINT32 flags, 
	void *pwidget
	);

//! Create an instance of an elastic point tool.
MXTHANDLE mxtEPoint (
	UINT32 flags, 
	void *pwidget
	);

//! Create elastic polyline tool.
MXTHANDLE mxtEPolyLine (
	UINT32 flags, 
	void *pwidget
	);

//! Create elastic region tool.
MXTHANDLE mxtERegion (
	UINT32 flags, 
	void *pwidget
	);

//! Create an instance of an elastic regpoly tool.
MXTHANDLE mxtERegPoly (
	UINT32 flags, 
	void *pwidget
	);

//! Create an instance of an elastic text label tool.
MXTHANDLE mxtETextLabel (
	UINT32 flags, 
	void *pwidget
	);

//! Create an instance of point selection tool.
MXTHANDLE mxtSelectPoint (
	UINT32 flags, 
	void *pwidget
	);

//! Add callback to tool.
//!
//! Valid reasons:		The reasons are in reason classes		
//!	<ul>
//! 	<li>MxtCR_IsBegin
//!		<li>MxtCR_IsContinue		
//!		<li>MxtCR_IsEnd		
//!		<li>MxtCR_IsOther		
//!		<li>MxtCR_IsActivate
//!		<li>MxtCR_IsDestroy
//!		<li>MxtCR_IsOther reasons:			
//!			<ul>
//!			<li>MxtCR_NoReason
//!			<li>MxtCR_SetValues			
//!			<li>MxtCR_NewLimits
//!			<li>MxtCR_SelectPosition		SelectPoint tool sends this on a left button click
//!			<li>MxtCR_Manage			
//!			<li>MxtCR_Unmanage			
//!			<li>MxtCR_ClearPosition
//!			<li>MxtCR_Cancel				User pressed 'Cancel' button
//!			</ul>
//!		<li>MxtCR_IsActivate reasons:			
//!			<ul>
//!			<li>MxtCR_Activate		
//!			</ul>
//!		<li>MxtCR_IsDestroy reasons:
//!			<ul>
//!			<li>MxtCR_Destroy		
//!			</ul>
//!		<li>MxtCR_IsBegin reasons:			
//!			<ul>
//!			<li>MxtCR_CreateBegin			
//!			<li>MxtCR_MoveBegin
//!			<li>MxtCR_ResizeBegin			
//!			<li>MxtCR_RotateBegin			
//!			<li>MxtCR_ShearBegin			
//!			<li>MxtCR_EditBegin
//!			</ul>
//!		<li>MxtCR_IsContinue reasons:			
//!			<ul>
//!			<li>MxtCR_MoveContinue			
//!			<li>MxtCR_ResizeContinue
//!			<li>MxtCR_RotateContinue			
//!			<li>MxtCR_ShearContinue			
//!			<li>MxtCR_EditContinue
//!			<li>MxtCR_CreateContinue		
//!			</ul>
//!		<li>MxtCR_IsEnd reasons:			
//!			<ul>
//!			<li>MxtCR_MoveEnd			
//!			<li>MxtCR_ResizeEnd
//!			<li>MxtCR_RotateEnd			
//!			<li>MxtCR_ShearEnd			
//!			<li>MxtCR_EditEnd			
//!			<li>MxtCR_CreateEnd
//!			</ul>
//!	 </ul>
int MxtAddCallback (
	MXTHANDLE vmxt, 							//!< Add callback to tool 
	McbCallbackFunc func, 					//!< Callback function
	void *udata, 								//!< User data passed to func
	UINT32 reason								//!< Reason set flags to get callbacks on
	);

//! Add callback to tool with position determined by priority.
int MxtCallbackAddExt (
	MXTHANDLE vmxt, 							//!< Add callback to tool
	McbCallbackFunc func, 					//!< Callback function
	void *udata, 								//!< User data passed to func
	UINT32 reason, 							//!< Reason set flags to get callbacks on
	INT8 priority
	);

//! Call "activate" callback for specified tool 
void MxtCallActivateCallback (
	MXTHANDLE mxt								//!< Tool handle
	);

//! Call "change" callback for specified tool.
void MxtCallChangeCallback (
	MXTHANDLE mxt, 							//!< Tool handle
	UINT32 reason								//!< Callback reason
	);

//! Clear tool position (erasing tool).
//!
//! This function undraws the tool then clears the tool information.
//!	Useful in activate callbacks to simulate adding elements.
void MxtClearPosition (
	MXTHANDLE mxt								//!< Handle returned by MxtCreate()
	);

//! Create new tool instance. 
//!
//! This function creates a tool handle by calling the appropriate tool
//!	create function.  It also sets up the default tool position which will
//!	be a clear tool (no info).  Neither the tool create function nor this
//!	function will create the dialog.  Any setup of the tool dialog must be
//!	done before MxtManage() is called because MxtManage() sets up the dialog.
//!	NOTE:	Call MxtSetValues() to setup default tool information.
//!	List of flags to pass in this function (see MxtSetFlags()).
//!	  \li \b	MxtFLAG_EnableManual		
//!	  \li \b	MxtFLAG_EnableDialog		
//!	  \li \b	MxtFLAG_EnableXYDig
//!	  \li \b	MxtFLAG_EnableSnapGrid
//!		Any tool specific flags dealing with dialog management.  Eg:
//!			  \li \b MxEPOLYLINE_NoEditZValue			
//!			  \li \b MxEPOLYLINE_NoClosePolyB
MXTHANDLE MxtCreate (
	Widget pwidget, 									//!< Drawing Area widget
	MXTHANDLE (*CreateFunc)(UINT32, void*), 	//! Function to create specific tool
	UINT32 flags										//!< MxtFLAG... flags
	);

//! Destroy previously created tool instance.
//!
//! This function does basic cleanup of the tool.  It will unmanage the tool
//!	if not done so already.  It will also remove the tool tag string to keep
//!	the tool pointer from being used as a tool.  It will free the pointer.
void MxtDestroy (
	MXTHANDLE mxt								//!< Tool handle
	);

//! Draw tool graphic. 
void MxtDrawGraphic (
	MXTHANDLE vmxt, 							//!< Handle returned by MxtCreate()
	const LRECT2D *rect						//!< Area to draw into, NULL for the whole area
	);

//! Display dialog allowing user to edit global tool colors and width. 
//!
//!	@return 0 if user changed values, EUserCancel if cancelled.
//! 
//! This will automatically update the process global tool settings and save to the Ini file.
//!	The application should refresh all active tools if possible using MxtSetToolColor() for
//!	each DrawingArea's current tool.
int MxtEditGlobalOptions (
	Widget pwidget								//!< Parent for dialog
	);

//! GPS input is enabled for this view.
void MxtEnableGps (
	Widget pwidget, 							//!< Drawing Area widget
	int enabled									//!< TRUE if GPS is set for this drawing area
	);

//! Free tool structure internal data storage (Eg. EPOLYLINEINFO).
//!
//! This function frees any allocated data associated with the tool
//!	structure.  Eg. the point buffer for line data.  It will not free
//!	the data associated with a change callback.
void MxtFreeValues (
	MXTHANDLE mxt, 							//!< Handle returned by MxtCreate()
	void *toolvalues							//!< Pointer to tool structure
	);

//! Generate manual form for tool if it exists.
//!
//! @return Form widget or NULL if non-existant.
//!
//! NOTE: Must specify MxtFLAG_NoDialog to use this function.
Widget MxtGenerateManualForm (
	MXTHANDLE vmxt, 							//!< Handle returned by MxtCreate()
	Widget parent								//!< Parent for main tool form, assumed to be a RowColumn
	);

//! Generate a region form the tool if supported.
//!
//! @return 1 if a region is generated, 0 if not, < 0 Error.
//!
//! This function will call MregionInit().  If an error occurs, MregionFree() is called.
int MxtGenerateRegion (
	MXTHANDLE vmxt, 							//!< Handle returned by MxtCreate()
	REGION2D& region,							//!< The region to fill in
	UINT32 flags								//!< May be used at some point
	);

//! Generate spatial form for tool if it exists.
//!
//! @return Form widget or NULL if non-existant.
//!
//! NOTE: Must specify MxtFLAG_NoDialog to use this function.
Widget MxtGenerateSpatialForm (
	MXTHANDLE vmxt, 							//!< Handle returned by MxtCreate()
	Widget parent								//!< Parent for main tool form, assumed to be a RowColumn
	);

//! Generate main tool form for tool if it exists.
//!
//! @return Form widget or NULL if non-existant.
//!
//! NOTE: Must specify MxtFLAG_NoDialog to use this function.
Widget MxtGenerateToolForm (
	MXTHANDLE vmxt, 							//!< Handle returned by MxtCreate()
	Widget parent								//!< Parent for main tool form, assumed to be a RowColumn
	);

//! Return current arrow key delta values.
//!
//! This function gets the amount of movement for hitting arrow keys in 
//!	object coordinates (i.e. deltas).
void MxtGetArrowKeyDeltas (
	MXTHANDLE mxt, 							//!< Handle returned by MxtCreate()
	double *delta_x, 							//!< Length to move in X direction (RETURNED)
	double *delta_y							//!< Length to move in Y direction (RETURNED)
	);

//! Return current tool boundary rectangle.
//!
//! @return 1 - Tool has boundaries, 0 - Tool can move off drawable.
//!
//! This function returns the boundaries for the tool.
int MxtGetBounds (
	MXTHANDLE mxt, 							//!< Handle returned by MxtCreate()
	DRECT2D *rect								//!< Rect containing coordinates to bound, tool coords
	);

//! Return currently active tool handle.
//!
//! @return Handle to the currently active tool or NULL if no tool is active.
MXTHANDLE MxtGetCurrentTool (
	MXTHANDLE mxt								//!< Can either be a MXTHANDLE or the parent widget of the tool
	);

//! Return current tool flags.
//!
//! This function returns the current flags set in the tool.  You should call
//!	this function to get the current flags, set or unset certain flags, then
//!	call MxtSetFlags() to set the flags.
UINT32 MxtGetFlags (
	MXTHANDLE mxt
	);

//! Return global colors set for tools in the application.
void MxtGetGlobalColor (
	int *UseTool, 								//!< Not used, can pass NULL
	COLOR *color, 								//!< Primary tool color RETURNED
	COLOR *seccolor, 							//!< Secondary tool color RETURNED
	int *LineWidth								//!< Tool line width RETURNED
	);

//! Return status of manual edit dialog (on/off).
//!
//! @return TRUE if the manual editor is in use, FALSE if not.
//!
//! This function returns the status of the manual editor mode.
int MxtGetManualMode (
	MXTHANDLE mxt								//!< Handle returned by MxtCreate()
	);

//! Return current tool manual parameter settings.
//!
//! This function retrieves the manual tool editing information within the tool.
void MxtGetManualParms (
	MXTHANDLE mxt, 							//!< Handle returned by MxtCreate()
	MANUALTOOLPARMS *mtparms				//!< Pointer to manual tool editing parameters
	);

//! Return current tool snap grid settings.
//!
//! This function retrieves the snap grid information from the tool. 
void MxtGetSnapGrid (
	MXTHANDLE mxt, 							//!< Handle returned by MxtCreate()	
	SNAPGRIDPARMS *snapgrid					//!< Pointer to snap grid parameters
	);

//! Return current status of tool snap grid (on/off).
//!
//! @return TRUE if the snap grid is in use, FALSE if not.
//!
//! This function returns the status of the snap grid mode.
int MxtGetSnapGridMode (
	MXTHANDLE mxt								//!< Handle returned by MxtCreate()
	);

//! Return the GC (X drawing parameter handle) of the tool.
GC MxtGetToolGC (
	MXTHANDLE mxt
	);

//! Get size of tool structure (Eg. sizeof(EBOXINFO)).
//!
//! Returns the structure size of the tool.
int MxtGetToolSize (
	MXTHANDLE mxt								//!< Handle returned by MxtCreate()
	);

//! Get tool/window transformation.
void MxtGetTrans2D (
	MXTHANDLE mxt, 							//!< Tool handle
	TRANS2D_AFFINE& TtoW						//!< Tool-to-Window transformation returned
	);

//! Get "userdata" resource for a tool.
//!
//! Returns the pointer to the data associated with the tool.  Set by MxtSetUserData().
void *MxtGetUserData (
	MXTHANDLE mxt								//!< Handle returned by MxtCreate()
	);

//! Get current tool-specific "values" structure.
//!
//! The tool makes a copy of the information and places it in the toolvalues
//!	structure.  No direct access of the tool data is allowed.  You must free
//!	any allocated fields in the tool structure using MxtFreeValues().
//!	Definitions of tool structures is with the respective "Create"
//!	function for that tool.  Eg: MxtCreateEBox has a definition of an
//!	EBOXINFO structure.
int MxtGetValues (
	MXTHANDLE mxt, 							//!< Handle returned by MxtCreate()
	void *toolvalues							//!< Pointer to tool structure to copy into
	);

//! Return current status of X-Y Digitizer (on/off).
//!
//! @return TRUE if the digitizer is in use, FALSE if not.
int MxtGetXYDigitizerMode (
	MXTHANDLE mxt								//!< Handle returned by MxtCreate()
	);

//! Return "position" status of tool.
//! 
//! @return TRUE if tool has graphic position set.
int MxtHasPosition (
	MXTHANDLE mxt								//!< Handle returned by MxtCreate()
	);

//! Inform caller if we are in an update background call.
int MxtInBgUpdateCall (
	MXTHANDLE mxt
	);

//! Return "managed" status of tool.
//!
//! @return TRUE if tool has been managed.  Set by MxtManage().
int MxtIsManaged (
	MXTHANDLE mxt								//!< Handle returned by MxtCreate()
	);

//! Return "suspended" status of tool.
//!
//! @return TRUE if tool has been suspended.  Set by MxtSuspend(). 
int MxtIsSuspended (
	MXTHANDLE mxt								//!< Handle returned by MxtCreate()
	);

//! Check to see if GPS tracking is on.  
int MxtIsTrackingGps (
	MXTHANDLE mxt
	);

//! Make tool instance active, allowing user to modify it.
//!
//! This function creates the GC and tool dialog.  It also sets up specific
//!	event handlers to check for certain events, displays the tool if the
//!	tool has info, and sets up XYDigitizer stuff.  This should be called
//!	only if changing from another tool to this one, not to hide the tool if
//!	drawing on the drawable.  Use MxtSuspend/MxtResume() to accomplish tool
//!	graphic hiding.
void MxtManage (
	MXTHANDLE mxt								//!< Handle returned by MxtCreate()
	);

//! Set view projection of manual tool section.
int MxtManualViewCoordRefSysSet (
	MXTHANDLE vmxt, 												//!< Handle returned by MxtCreate()
	const SPATREF::COORDREFSYS& ViewCoordRefSys,			//!< Projection information
	bool ForceProj													//!< Force the projection even if the user has already changed it
	);

//! Move tool under program control.
//!
//! This function shifts the tool values.  This function is optionally supported by the tool.
//!
//!	Adjustment flags:
//!		\li \b MxtADJUST_Relative		Make adjustments relative to current position (default)
//!		\li \b MxtADJUST_Absolute		Move to new absolute coordinates specified
//!		\li \b MxtADJUST_ToolCoords		Make adjustments in "tool" coordinates (default)
//!		\li \b MxtADJUST_ScreenCoords	Make adjustments in screen coordinates
//!		\li \b MxtADJUST_Notify			Call ChangeCallback after adjustments made
void MxtMove (
	MXTHANDLE mxt, 							//!< Handle returned by MxtCreate()
	double dx, 									//!< Delta x
	double dy, 									//!< Delta y
	XAnyEvent *event, 						//!< Event that caused the move, can be NULL
	UINT32 flags								//!< MxtADJUST... flags
	);

//! Raise tool dialog window to the foreground.
void MxtRaiseWindow (
	MXTHANDLE vmxt								//!< Handle returned by MxtCreate()
	);

//! Remove callback from tool. 
void MxtRemoveCallback (
	MXTHANDLE vmxt, 
	McbCallbackFunc func, 
	void *udata, 
	UINT32 reason
	);

//! Resize tool under program control.
//!
//! This function scales the tool values.  This function is optionally supported by the tool.
//!
//!	Adjustment flags:
//!		\li \b MxtADJUST_Relative		Make adjustments relative to current size (default)
//!		\li \b MxtADJUST_Absolute		Change to new size specified
//!		\li \b MxtADJUST_ToolCoords		Make adjustments in "tool" coordinates (default)
//!		\li \b MxtADJUST_ScreenCoords	Make adjustments in screen coordinates
//!		\li \b MxtADJUST_Notify			Call ChangeCallback after adjustments made
void MxtResize (
	MXTHANDLE mxt, 							//!< Handle returned by MxtCreate()
	double dx, 									//!< Scale x
	double dy, 									//!< Scale y
	XAnyEvent *event, 						//!< Event that caused the move, can be NULL
	UINT32 flags								//!< MxtADJUST... flags
	);

//! Resume (redraw graphic if possible) the tool.
//!
//! This function resumes the tool from a MxtSuspend(). 
void MxtResume (
	MXTHANDLE mxt								//!< Handle returned by MxtCreate()
	);

//! Rotate tool under program control.
//!
//! This function rotates the tool values.  This function is optionally
//!	supported by the tool.  Rotation angle is in RADIANS.	
//!
//!	Adjustment flags:
//!		\li \b MxtADJUST_Relative		Make adjustments relative to current rotation angle (default)
//!		\li \b MxtADJUST_Absolute		Change to new rotation angle specified
//!		\li \b MxtADJUST_ToolCoords		Make adjustments in "tool" coordinates (default)
//!		\li \b MxtADJUST_ScreenCoords	Make adjustments in screen coordinates
//!		\li \b MxtADJUST_Notify			Call ChangeCallback after adjustments made
void MxtRotate (
	MXTHANDLE mxt, 
	double drot, 
	XAnyEvent *event, 
	UINT32 flags
	);

//! Save global color setting to tntproc.ini file.
void MxtSaveGlobalColor (
	char *group
	);

//! Set function to be called when an "activate" event occurs.
//!
//! This function sets the activate callback for when the user presses return,
//!	right mouse button, or if a dialog exists, "Save" button is pressed.  Most
//!	other Mxt...() functions are callable from this callback, except:
//!	MxtDestroy(), MxtManage(), and MxtUnmanage().
//!
//! Callback "reasons"		
//!		MxtCR_Activate			An activate event occured
//! 
//!	An "activate" event occurs when the user presses Button3 (usually right button) or \<ENTER\> while 
//! the parent widget has keyboard focus.
void MxtSetActivateCallback (
	MXTHANDLE mxt, 
	void (*CBFunc)(MxtAnyCallbackStruct*, void*), 
	void *udata
	);

//! Set deltas for arrow key handling.
//!
//! This function sets the amount of movement for hitting arrow keys in 
//!	object coordinates (i.e. deltas). 
void MxtSetArrowKeyDeltas (
	MXTHANDLE mxt, 							//!< Handle returned by MxtCreate()
	double delta_x, 							//!< Length to move in X direction
	double delta_y								//!< Length to move in Y direction
	);

//! Set the update background function.
void MxtSetBgUpdateFunc (
	Widget pwidget, 
	void (*UpdBg)(Widget, DRECT2D*, void*), 
	void *udata
	);

//! Set bounding rectangle for the tool.
//!
//! This function sets boundaries for the tool.  This function will recognize a 
//! MxtFLAG_CanMoveOff flag.
//!
//! List of flags to pass in this function.		
//!		\li \b MxtBOUND_Xmin		
//! 	\li \b MxtBOUND_Ymin
//!		\li \b MxtBOUND_Xmax		
//!		\li \b MxtBOUND_Ymax
//!		\li \b MxtBOUND_FullRect = (MxtBOUND_Xmin|MxtBOUND_Ymin|MxtBOUND_Xmax|MxtBOUND_Ymax)
//!		MxtFLAG_CanMoveOff
void MxtSetBounds (
	MXTHANDLE mxt, 							//!< Handle returned by MxtCreate()	
	DRECT2D *rect, 							//!< Rect containing coordinates to bound, object coords
	UINT32 flags								//!< Which values are set to bound
	);

//! Set function to be called when any tool position values are changed.
//!
//! This function sets the change callback for the program to get any changes
//!	done to the tool.  You are allowed to set multiple callbacks for the tool
//!	and can be removed via a call to MxtRemoveChangeCallback().
//!
//! Callback "reasons":
//!		\li \b MxtCR_NoReason			For the hell of it
//!		\li \b MxtCR_Activate			An activate event occured
//!		\li \b MxtCR_SetValues		MxtSetValues() function was invoked
//!		\li \b MxtCR_NewLimits		New limits have been set which affected tool position
//!		\li \b MxtCR_SelectPosition	The selection tool has selected a position
//!		\li \b MxtCR_Manage			Tool has been managed		
//!		\li \b MxtCR_Unmanage			Tool has been unmanaged
//!		\li \b MxtCR_ClearPosition	Tool internal data is cleared
//!		\li \b MxtCR_MoveBegin		Move operation beginning
//!		\li \b MxtCR_MoveContinue	Move operation continuing
//!		\li \b MxtCR_MoveEnd			Move operation has ended
//!		\li \b MxtCR_ResizeBegin		Resize operation beginning
//!		\li \b MxtCR_ResizeContinue	Resize operation continuing
//!		\li \b MxtCR_ResizeEnd		Resize operation has ended
//!		\li \b MxtCR_RotateBegin		Rotate operation beginning
//!		\li \b MxtCR_RotateContinue	Rotate operation continuing
//!		\li \b MxtCR_RotateEnd		Rotate operation has ended
//!		\li \b MxtCR_ShearBegin		Shear operation beginning
//!		\li \b MxtCR_ShearContinue	Shear operation continuing
//!		\li \b MxtCR_ShearEnd			Shear operation has ended
//!		\li \b MxtCR_EditBegin		General editing operation beginning
//!		\li \b MxtCR_EditContinue	General editing operation continuing	
//!		\li \b MxtCR_EditEnd			General editing operation has ended	
//!		\li \b MxtCR_CreateBegin		User is "creating" new "tool"
//!		\li \b MxtCR_CreateContinue	User is in the process of "creating" new "tool"
//!		\li \b MxtCR_CreateEnd		User is done "creating" new "tool"
void MxtSetChangeCallback (
	MXTHANDLE mxt, 											//!< Handle returned by MxtCreate()
	void (*CBFunc)(MxtAnyCallbackStruct*, void*), 	//! Change callback function
	void *udata													
	);

//! Set the position of the dialog.
int MxtSetDialogPosition (
	MXTHANDLE vmxt, 
	int NewPosn
	);

//! Set flags for the tool.
//!
//! This function REPLACES the current flags set in the tool.  You should call
//!	MxtGetFlags() to get the current flags, set or unset certain flags, then
//!	call this function to set the flags.
//!
//!	NOTE: Certain dialog management flags will work only if set before
//!	a call to MxtManage().  The best place to pass them is in MxtCreate().
//!	The flags in question are listed with MxtCreate().
void MxtSetFlags (
	MXTHANDLE mxt, 							//!< Handle returned by MxtCreate()
	UINT32 flags								//!< Tool flags
	);

//! Set color and width for all tools in the application. 
void MxtSetGlobalColor (
	Widget parent, 							//!< Parent widget that tools are attached to
	int UseTool, 								//!< Turn on solid tools if 1, else 0
	const COLOR& color, 								//!< Color to use
	const COLOR& seccolor, 							
	int LineWidth								//!< Line width to use
	);

//! Set tool GPS source.
void MxtSetGpsSource (
	MXTHANDLE vmxt, 							//!< Handle returned by MxtCreate()
	void *vNewSource							//!< GPS Source handle, NULL to turn off
	);

//! Set help string for tool dialog, overrides default help.
//!
//! This function replaces the default help string with the one passed in.
//!	For correctness, the help that is to replace this help needs to have a
//!	SEE ALSO to the default help for the tool.
//!
//!	This function must be called before MxtManage().
void MxtSetHelpString (
	MXTHANDLE mxt, 							//!< Handle returned by MxtCreate()
	char *helpstr								//!< Help string to replace default string
	);

//! Set status of manual editing dialog (on/off).
//!
//! This function turns on/off the manual editor mode of the tool.  If the
//!	tool has a dialog and the correct flag is set, the tool dialog will have
//!	a manual editor button on the dialog.  If not, this function must	be used
//!	to envoke manual editor mode.
//!	NOTE: The manual editor doubles as a tool coordinate reporter.  The update
//!	of the manual editor is handled by the tool itself.  The tool management
//!	layer only handles opening, closing and managing of the manual dialog.
void MxtSetManualMode (
	MXTHANDLE mxt, 							//!< Handle returned by MxtCreate()	
	int setflag									//!< On/Off
	);

//! Set manual editing parameters for the tool.
//!
//! @return 0 or EBadFuncParm for the manual editing parameters being invalid.
//!
//! This function sets the manual tool editing information within the tool.
int MxtSetManualParms (
	MXTHANDLE mxt, 							//!< Handle returned by MxtCreate()
	MANUALTOOLPARMS *mtparms				//!< Pointer to manual tool editing parameters
	);

//! Set output transparm for manual entry and GPS.
//!
//! This function sets the internal transparms for both the GPS and manual entry
//!	which describes the Map projection to tool transformation.
int MxtSetOutputTrans (
	MXTHANDLE vmxt, 							//!< Handle returned by MxtCreate()	
	const TRANS2D_MAPGEN& trans
	);

//! Set snap grid parameters for the tool.
//!
//! @return 0 or EBadFuncParm for the snap grid info being invalid.
//!
//! This function sets the snap grid information within the tool. 
int MxtSetSnapGrid (
	MXTHANDLE mxt, 							//!< Handle returned by MxtCreate()	
	SNAPGRIDPARMS *snapgrid					//!< Pointer to snap grid parameters
	);

//! Set status of snap grid (on/off).
//!
//!	This function turns on/off the snap grid mode of the tool.  If the tool
//!	has a dialog and the correct flag is set, the tool dialog will have a
//!	snap grid toggle button on the dialog.  If not, this function must
//!	be used to toggle snap grid mode.
void MxtSetSnapGridMode (
	MXTHANDLE mxt, 							//!< Handle returned by MxtCreate()	
	int setflag									//!< On/Off
	);

//! Set color and width for the tool only.
void MxtSetToolColor (
	MXTHANDLE vmxt, 							//!< Handle returned by MxtCreate()	
	COLOR *color, 								//!< Primary color to use, NULL to set to global primary color
	COLOR *seccolor, 							//!< Secondary color to use, NULL to set to global secondary color
	int LineWidth								//!< Line width to use, -1 to set to global line width
	);

}	// extern "C"
//! Set tool/window transformation.
//!	NOTE: If this function is not called, the tool will assume a one-one
//!	mapping of the coordinates.
void MxtSetTrans2D (
	MXTHANDLE mxt, 							//!< Handle returned by MxtCreate()
	const TRANS2D_AFFINE& TtoW				//!< Tool-to-Window transformation
	);

extern "C" {

//! Set tool/window transformation.
//!
//!	NOTE: If this function is not called, the tool will assume a one-one
//!	mapping of the coordinates.
void MxtSetTrans2D (
	MXTHANDLE mxt, 							//!< Handle returned by MxtCreate()
	const MAT3X3 OtoD,						//!< Tool to Drawable transform
	const MAT3X3 DtoO							//!< Drawable to Tool transform
	);

//! Set "userdata" resource for a tool.
void MxtSetUserData (
	MXTHANDLE mxt, 							//!< Handle returned by MxtCreate()
	void *udata									//!< User data to set
	);

//! Set tool-specific "values" structure.
//!
//! The tool makes a copy of toolvalues and correctly sets up its internal
//!	information.  A error is returned if the tool data is invalid.
//!
//!	NOTE: If specific tool and/or generic flags need to be set before
//!	setting the tool, use MxtSetFlags() before calling this function. 
//!
//!	Definitions of tool structures is with the respective "Create"
//!	function for that tool.  Eg: MxtCreateEBox has a definition of an	EBOXINFO structure.
int MxtSetValues (
	MXTHANDLE mxt, 							//!< Handle returned by MxtCreate()
	void *toolvalues, 						//!< Pointer to tool structure to copy from
	int notify									//!< Call change callback if one is assigned
	);

//! Set edit dialog button row instead of default.
//!
//! This function must be called before MxtManage(). The MXTHANDLE is passed as the callback data.  
//! The passed in udata is set up as userdata on the widget.
void MxtSetWindowButtonRow (
	MXTHANDLE mxt, 							//!< Handle returned by MxtCreate()
	BUTTONITEM *buttonrow, 					//!< List of buttons to replace default buttonrow
	void *udata									//!< User data passed to MxCreateButtonRow as userdata
	);

//! Set function to allow programmer addition to edit dialog.
//!
//! This function must be called before MxtManage().
void MxtSetWindowFormCallback (
	MXTHANDLE mxt, 							//!< Handle returned by MxtCreate()
	int (*AddToForm)(Widget, void*), 	//! Function to create user specified form
	void *udata									//!< User data passed to AddToForm()
	);

//! Set edit dialog "save" button to somethine else.
//!
//! This function sets the "save" button to another string. This function must 
//! be called before MxtManage().
void MxtSetWindowSaveButton (
	MXTHANDLE mxt, 							//!< Handle returned by MxtCreate()
	const char *savebutton							//!< String that replaces "save" in dialog
	);


}		// Extern "C"

//! Set title for tool control window (if any).
//!
//! This function looks up the window title using the TEXTID
//! If not found, it will use the default. This function must be called before MxtManage().
void MxtSetWindowTitle (
	MXTHANDLE mxt, 							//!< Handle returned by MxtCreate()	
	TEXTID textid
	);

extern "C" {

//! Set pixel to XOR with the drawable.
//!
//! This function sets the XOR pixel to use in the GC to draw with.  Usually
//!	passed the background pixel of the drawable to handle different hued
//!	backgrounds.	This function must be called before MxtManage().
void MxtSetXORPixel (
	MXTHANDLE mxt, 							//!< Handle returned by MxtCreate()	
	Pixel pixel									//!< Pixel to use to XOR with
	);

//! Set status of X-Y Digitizer (on/off).
//!
//! This function turns on/off the digitizer mode of the tool.  If the tool
//!	has a dialog and the correct flag is set, the tool dialog will have a
//!	X-Y Digitizer toggle button on the dialog.  If not, this function must
//!	be used to toggle digitizer mode.
void MxtSetXYDigitizerMode (
	MXTHANDLE mxt, 							//!< Handle returned by MxtCreate()	
	int setflag									//!< On/Off
	);

//! Set translation function from X-Y Digitizer to tool coordinates.
//!
//! This function sets the digitizer to object translation function.
//!	If not set, the tool assumes a 1-1 mapping of the X-Y Digitizer 
//!	coordinates to object coordinates (i.e. trans2dinit()).
void MxtSetXYDtoObjectTrans (
	MXTHANDLE mxt, 							//!< Handle returned by MxtCreate()	
	TRANSFUNC tfunc, 							//!< X-Y Digitizer to Object coordinates
	void *udata									//!< User data passed to tfunc()
	);

//! Change tool shear angle under program control.
//!
//! This function shears the tool values.  This function is optionally
//!	supported by the tool.
//!
//!	Adjustment flags:
//!		\li \b MxtADJUST_Relative		Make adjustments relative to current shear angle (default)
//!		\li \b MxtADJUST_Absolute		Change to new shear angle specified
//!		\li \b MxtADJUST_ToolCoords		Make adjustments in "tool" coordinates (default)
//!		\li \b MxtADJUST_ScreenCoords	Make adjustments in screen coordinates
//!		\li \b MxtADJUST_Notify			Call ChangeCallback after adjustments made
void MxtShear (
	MXTHANDLE mxt, 							//!< Handle returned by MxtCreate()
	double dshear, 							//!< Shear angle
	XAnyEvent *event, 						//!< Event that caused the move, can be NULL
	UINT32 flags								//!< MxtADJUST... flags
	);

//! Suspend (undraw graphic if possible) the tool.
//!
//! This function suspends the tool (undraws the tool graphic) while
//!	manipulation of the drawing area is accomplished.  It will not affect
//!	any tool dialogs that are shown.
void MxtSuspend (
	MXTHANDLE mxt								//!< Handle returned by MxtCreate()
	);

//! Turn on GPS tracking.
void MxtTrackGps (
	MXTHANDLE vmxt, 
	int track
	);

//! Make tool instance inactive, erasing tool if necessary.
//!
//! This function removes the dialog and GC and event handlers, cleans up
//!	digitizer setup and undraws the tool if drawn.  This should be called
//!	only if changing from another tool to this one, not to hide the tool if
//!	drawing on the drawable.  Use MxtSuspend/MxtResume() to accomplish tool
//!	graphic hiding. 
void MxtUnmanage (
	MXTHANDLE mxt								//!< Handle returned by MxtCreate()
	);

}		// Extern "C"

//! Create crosshair tool.
//!
//! @return Tool handle (mxthandle) used in other Mxt... functions.
//!
//! Flags for structure initialization
//!		\li \b MxCROSSHAIR_BarLength		Bar lengths (hlen, vlen) set (not used yet)
//!		\li \b MxCROSSHAIR_NoBar0			Do not draw 0   degree bar
//!		\li \b MxCROSSHAIR_NoBar90			Do not draw 90  degree bar
//!		\li \b MxCROSSHAIR_NoBar180			Do not draw 180 degree bar
//!		\li \b MxCROSSHAIR_NoBar270			Do not draw 270 degree bar	Flags to control behavior
//!		\li \b MxtFLAG_NoRotate				Don't allow rotation
//!		\li \b MxtFLAG_CanMoveOff			Allow tool (center) to move off allowed area
inline MXTHANDLE MxtCreateCrossHair (
	Widget dawidget,							//!< DrawingArea widget to contain tool
	UINT32 flags = 0							//!< MxtFLAG_...
	) {
	return (MxtCreate(dawidget,mxtCrossHair,flags));
	}

//! Create elastic arc tool.
//!
//! @return Tool handle (mxthandle) used in other Mxt... functions.
//!
//! Flags for structure initialization:
//!		\li \b MxEARC_Rmin				Minimum radius set
//!		\li \b MxEARC_Rmax				Maximum radius set	
//!
//! Flags to control behavior:
//!		\li \b MxtFLAG_NoResize		Don't allow arc to be resized
//!		\li \b MxtFLAG_NoCreate		Don't allow new arc to be created
//!		\li \b MxtFLAG_CanMoveOff	Can move arc outside specified position ranges
inline MXTHANDLE MxtCreateEArc (
	Widget dawidget,							//!< DrawingArea widget to contain tool
	UINT32 flags = 0							//!< MxtFLAG_...
	) {
	return (MxtCreate(dawidget,mxtEArc,flags));
	}

//! Create elastic box tool.
//!
//! @return Tool handle (mxthandle) used in other Mxt... functions.
//!
//! Flags for structure initialization:
//!		\li \b MxEBOX_Range			Range fields (xinit, yinit, xlast, ylast) set
//!		\li \b MxEBOX_AllCorners		All corner locations set
//!		\li \b MxEBOX_Corner0			Corner 0, size and angle set
//!		\li \b MxEBOX_Corner1			Corner 1, size and angle set
//!		\li \b MxEBOX_Corner2			Corner 2, size and angle set
//!		\li \b MxEBOX_Corner3			Corner 3, size and angle set		
//!		\li \b MxEBOX_Wmin				Minimum width set
//!		\li \b MxEBOX_Hmin				Minimum height set		
//!		\li \b MxEBOX_Wmax				Maximum width set
//!		\li \b MxEBOX_Hmax				Maximum height set	
//!
//!	Flags to control behavior:
//!		\li \b MxtFLAG_NoResize		Don't allow box to be resized
//!		\li \b MxtFLAG_NoCreate		Don't allow new box to be created
//!		\li \b MxtFLAG_CanMoveOff	Can move box outside specified position ranges
//!		\li \b MxtFLAG_CanRotate		Don't allow box to be rotated
//!		\li \b MxEBOX_Proport			Make box proportional
inline MXTHANDLE MxtCreateEBox (
	Widget dawidget,							//!< DrawingArea widget to contain tool
	UINT32 flags = 0							//!< MxtFLAG_...
	) {
	return (MxtCreate(dawidget,mxtEBox,flags));
	}

//! Create elastic circle tool instance.
//!
//! @return Tool handle (mxthandle) used in other Mxt... functions.
//!
//! Flags for structure initialization:
//!		\li \b MxECIRCLE_Rmin			Minimum radius set
//!		\li \b MxECIRCLE_Rmax			Maximum radius set	
//!
//! Flags to control behavior:
//!		\li \b MxtFLAG_NoResize		Don't allow circle to be resized
//!		\li \b MxtFLAG_NoCreate		Don't allow new circle to be created
//!		\li \b MxtFLAG_CanMoveOff	Can move circle outside specified position ranges
inline MXTHANDLE MxtCreateECircle (
	Widget dawidget,							//!< DrawingArea widget to contain tool
	UINT32 flags = 0							//!< MxtFLAG_...
	) {
	return (MxtCreate(dawidget,mxtECircle,flags));
	}

//! Create elastic ellipse tool.
//!
//! @return Tool handle (mxthandle) used in other Mxt... functions.
//!
//! Flags for structure initialization:		
//!		\li \b MxEELLIPSE_Rmin 		Minimum radius set
//!		\li \b MxEELLIPSE_Rmax 		Maximum radius set	
//!		
//!	Flags to control behavior
//!		\li \b MxtFLAG_NoResize		Don't allow ellipse to be resized
//!		\li \b MxtFLAG_NoCreate		Don't allow new ellipse to be created
//!		\li \b MxtFLAG_CanMoveOff	Can move ellipse outside specified position ranges
inline MXTHANDLE MxtCreateEEllipse (
	Widget dawidget,							//!< DrawingArea widget to contain tool
	UINT32 flags = 0							//!< MxtFLAG_...
	) {
	return (MxtCreate(dawidget,mxtEEllipse,flags));
	}

//! Create elastic line tool instance.
//!
//! @return Tool handle (mxthandle) used in other Mxt... functions.
//!
//! Flags to control behavior
//!		\li \b MxtFLAG_NoResize		Don't allow line to be resized (length changed)
//!		\li \b MxtFLAG_NoCreate		Don't allow new line to be created
//!		\li \b MxtFLAG_CanMoveOff	Can move line outside specified position ranges
inline MXTHANDLE MxtCreateELine (
	Widget dawidget,							//!< DrawingArea widget to contain tool
	UINT32 flags = 0							//!< MxtFLAG_...
	) {
	return (MxtCreate(dawidget,mxtELine,flags));
	}

//! Create elastic multi-point tool.
//!
//! @return Tool handle (mxthandle) used in other Mxt... functions.
//!
//! Flags to control behavior
//!		\li \b MxEMULITPOINT_NoEditZValue		Don't let user edit z value
//!		\li \b MxEMULTIPOINT_SinglePointOnly	Don't allow multi-point editing
inline MXTHANDLE MxtCreateEMultiPoint (
	Widget dawidget,							//!< DrawingArea widget to contain tool
	UINT32 flags = 0							//!< MxtFLAG_...
	) {
	return (MxtCreate(dawidget,mxtEMultiPoint,flags));
	}

//! Create elastic multi-line/polygon tool.
//!
//! @return Tool handle (mxthandle) used in other Mxt... functions.
//!
//! Flags to control behavior:
//!		\li \b MxEMULTILINE_EditPoly				Edit polygon
//!		\li \b MxEMULTILINE_EnableResetB			Enable tool reset button
//!		\li \b MxEMULTILINE_NoClearB				Don't show clear button
//!		\li \b MxEMULTILINE_NoClosePolyB			Don't show close poly button
//!		\li \b MxEMULTILINE_UseCurveFit			Edit a curve fit
//!		\li \b MxEMULTILINE_CanChgLinePolyMode	User can toggle between polygon and line modes
//!		\li \b MxEMULTILINE_NoEditZValue			Don't let user edit z value
//!		\li \b MxEMULTILINE_SingleLineOnly		Don't allow multi-line editing
inline MXTHANDLE MxtCreateEMultiPolyLine (
	Widget dawidget,							//!< DrawingArea widget to contain tool
	UINT32 flags = 0							//!< MxtFLAG_...
	) {
	return (MxtCreate(dawidget,mxtEMultiPolyLine,flags));
	}

//! Create "point" tool.
//! 
//! @return Tool handle (mxthandle) used in other Mxt... functions.
inline MXTHANDLE MxtCreateEPoint (
	Widget dawidget,							//!< DrawingArea widget to contain tool
	UINT32 flags = 0							//!< MxtFLAG_...
	) {
	return (MxtCreate(dawidget,mxtEPoint,flags));
	}

//! Create elastic "polyline/polygon" tool.
//!
//! @return Tool handle (mxthandle) used in other Mxt... functions.
//!
//! Flags to control behavior
//!		\li \b MxtFLAG_NoResize						Don't allow line to be resized (length changed)
//!		\li \b MxtFLAG_NoCreate						Don't allow new line to be created
//!		\li \b MxtFLAG_CanMoveOff					Can move line outside specified position ranges
//!		\li \b MxEPOLYLINE_EditPoly					Edit polygon
//!		\li \b MxEPOLYLINE_UseSnapCoord			Use the snap coord callback reason to allow snapping vertices
//!		\li \b MxEPOLYLINE_EnableResetB			Enable tool reset button
//!		\li \b MxEPOLYLINE_NoClosePolyB			Don't show close poly button
//!		\li \b MxEPOLYLINE_UseCurveFit				Edit a curve fit
//!		\li \b MxEPOLYLINE_CanChgLinePolyMode	User can toggle between polygon and line modes
//!		\li \b MxEPOLYLINE_NoEditZValue			Don't let user edit z value
inline MXTHANDLE MxtCreateEPolyLine (
	Widget dawidget,							//!< DrawingArea widget to contain tool
	UINT32 flags = 0							//!< MxtFLAG_...
	) {
	return (MxtCreate(dawidget,mxtEPolyLine,flags));
	}

//! Create "region" display tool.
//! 
//! Tool handle (mxthandle) used in other Mxt... functions.
//!
//! Flags to control behavior:
//!		\li \b MxEREGION_NoMove
inline MXTHANDLE MxtCreateERegion (
	Widget dawidget,							//!< DrawingArea widget to contain tool
	UINT32 flags = 0							//!< MxtFLAG_...
	) {
	return (MxtCreate(dawidget,mxtERegion,flags));
	}

//! Create elastic "regular polygon" tool.
//!
//! @return Tool handle (mxthandle) used in other Mxt... functions.
//!
//! Flags to control behavior:
//!		\li \b MxEREGPOLY_Rmin		
//!		\li \b MxEREGPOLY_Rmax
inline MXTHANDLE MxtCreateERegPoly (
	Widget dawidget,							//!< DrawingArea widget to contain tool
	UINT32 flags = 0							//!< MxtFLAG_...
	) {
	return (MxtCreate(dawidget,mxtERegPoly,flags));
	}

//! Create elastic "text label" tool.
//!
//! @return Tool handle (mxthandle) used in other Mxt... functions.
//!
//! Flags to control behavior
//!		\li \b MxtFLAG_NoResize				Don't allow line to be resized (length changed)
//!		\li \b MxtFLAG_NoCreate				Don't allow new line to be created
//!		\li \b MxtFLAG_CanMoveOff			Can move line outside specified position ranges
//!		\li \b MxETEXTLABEL_NoLeaderLines	Do not allow editing of leader lines
//!		\li \b MxETEXTLABEL_NoAdvanced		Do not allow advanced style button
//!		\li \b MxETEXTLABEL_NoBaselineEdit	Do not allow baseline to be edited
//!		\li \b MxETEXTLABEL_NoStylePanel		Do not allow the style panel to be created
inline MXTHANDLE MxtCreateETextLabel (
	Widget dawidget,							//!< DrawingArea widget to contain tool
	UINT32 flags = 0							//!< MxtFLAG_...
	) {
	return (MxtCreate(dawidget,mxtETextLabel,flags));
	}

//! Create point selection tool instance.
//!
//! @return Tool handle (mxthandle) used in other Mxt... functions.
inline MXTHANDLE MxtCreateSelectPoint (
	Widget dawidget,							//!< DrawingArea widget to contain tool
	UINT32 flags = 0							//!< MxtFLAG_...
	) {
	return (MxtCreate(dawidget,mxtSelectPoint,flags));
	}

//! Return "reason class" for specified callback "reason".
//!
//! @return Reason class or 0 if reason is invalid. 
//!
//! 	Class							Description	
//!	\li \b MxtCR_IsOther		Reason not in one of following classes
//!	\li \b MxtCR_IsActivate	Reason is an activate reason
//!	\li \b MxtCR_IsDestroy	Reason is an destroy reason	
//!	\li \b MxtCR_IsBegin		"Begin change"
//!	\li \b MxtCR_IsContinue	"Continue change"	
//!	\li \b MxtCR_IsEnd			"End change"	
//!	
inline UINT32 MxtGetReasonClass (
	UINT32 reason								//!< Callback reason (usually from cbdata->reason)
	) {
	return (reason & MxtCR_ReasonMask);
	}

//!@}

#endif		//!<  #ifdef INC_MI32_MXTOOLS_H 
