/**
 * \file mi32/triangulator.h
 * \brief Definitions for TRIANGULATOR
 *
 * \if NODOC
 * $Id: triangulator.h_v 1.3 2011/10/03 14:46:17 mju Exp $
 *
 * $Log: triangulator.h_v $
 * Revision 1.3  2011/10/03 14:46:17  mju
 * Remove tripleint32 struct as not used.
 *
 * Revision 1.2  2010/11/10 21:41:19  mju
 * Remove decompose using vertex indices.
 *
 * Revision 1.1  2010/11/04 21:00:55  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_TRIANGULATOR_H
#define	INC_MI32_TRIANGULATOR_H

#ifndef	INC_MI32_POLYLINE_H
#include <mi32/polyline.h>
#endif

#ifdef GEOMDLL
	#define CLASSEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSEXPORT MI_DLLCLASSIMPORT
#endif


#ifndef	GENERATING_DOXYGEN_OUTPUT
// Forward declarations.
class SUBREGION2D;
#endif

//! Perform triangular decomposition of geometric elements.
//! Retains internal work buffers for efficient reuse on multiple elements, avoiding reallocation where possible.
class CLASSEXPORT TRIANGULATOR {
	public:

		TRIANGULATOR ();
		~TRIANGULATOR ();

		//! Decompose closed polyline, returning vertex points.
		ERRVALUE Decompose (
			const POLYLINE& polyline,
			DOUBLE_ARRAY<DTRIANGLE2D>& triangles		//!< Triangle vertex points returned, in clockwise order
			);

		//! Decompose SUBREGION2D, returning vertex points.
		ERRVALUE Decompose (
			const SUBREGION2D& subregion,
			DOUBLE_ARRAY<DTRIANGLE2D>& triangles		//!< Triangle vertex points returned
			);

	private:
		#ifndef	GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;

		// Disallowed
		TRIANGULATOR (const TRIANGULATOR&);
		TRIANGULATOR& operator= (const TRIANGULATOR&);
		#endif
	};

#undef CLASSEXPORT

#endif	// INC_MI32_TRIANGULATOR_H
