/**
 * \file soa/owsexception.h
 * \brief SOA::OWS "exception" definitions
 *
 * \if NODOC
 * $Id: owsexception.h_v 1.1 2007/02/16 21:35:41 mju Exp $
 *
 * $Log: owsexception.h_v $
 * Revision 1.1  2007/02/16 21:35:41  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SOA_OWSEXCEPTION_H
#define	INC_SOA_OWSEXCEPTION_H

#ifndef	INC_MI32_MISTRING_H
	#include <mi32/mistring.h>
#endif

#ifndef	INC_MI32_STRUTF8_H
	#include <mi32/strutf8.h>
#endif

#ifndef	INC_MI32_MILIST_H
	#include <mi32/milist.h>
#endif

class HTTPMESSAGE;
class XMLNODE;

namespace SOA {
namespace OWS {

//=====================================================================================================================

//! OpenGeospatial Web Services Exception Report container.
//! A single report may contain any number of exception entries, though usually will contain exactly one.
class EXCEPTIONREPORT {
	public:

		//! Individual exception, contains message and possibly code and/or locator.
		class EXCEPTION {
			public:

				EXCEPTION ();

				~EXCEPTION ();

				//! Format message for display to user.
				MISTRING FormatMessage (
					) const;

				//! Get exception code, may be empty string.
				const MISTRING& GetCode (
					) const { return (m_Code); }

				//! Get locator, may be empty string.
				const MISTRING& GetLocator (
					) const { return (m_Locator); }

				//! Get message.
				const MISTRING& GetMessage (
					) const { return (m_Message); }

				//! Parse values from XMLNODE.
				ERRVALUE ParseXML (
					const XMLNODE *xmlnode
					);

			private:
				#ifndef	GENERATING_DOXYGEN_OUTPUT
				MISTRING m_Code;
				MISTRING m_Locator;
				MISTRING m_Message;
				#endif
			};

		typedef MILIST<EXCEPTION> EXCEPTIONLIST;

		//! Constructor.
		EXCEPTIONREPORT ();

		//! Destructor.
		~EXCEPTIONREPORT ();

		//! Format all exception messages for display to user.
		MISTRING FormatMessages (
			) const;

		//! Get exception list.
		const EXCEPTIONLIST& GetExceptionList (
			) const { return (m_ExceptionList); }

		//! Get original message text.
		const STRUTF8& GetOriginal (
			) const { return (m_OriginalMsg); }

		//! Get "version" attribute.
		const MISTRING& GetVersion (
			) const { return (m_Version); }

		//! Parse message from server.
		//! This will support several different XML message formats, including WMS 1.0.0, WMS 1.1.1, WMS 1.3.0, OWS 1.1.0.
		//! MIME type will typically be "application/vnd.ogc.se_xml" or just "text/xml".
		ERRVALUE ParseMessage (
			const HTTPMESSAGE& message					//!< Message to parse
			);

	private:
		#ifndef	GENERATING_DOXYGEN_OUTPUT
		MISTRING m_Version;
		EXCEPTIONLIST m_ExceptionList;
		STRUTF8 m_OriginalMsg;
		// Unimplemented
		EXCEPTIONREPORT (const EXCEPTIONREPORT& rhs);
		EXCEPTIONREPORT& operator= (const EXCEPTIONREPORT& rhs);
		#endif
	};

//=====================================================================================================================

}	// End namespace OWS
}	// End namespace SOA

#endif	// INC_SOA_OWSEXCEPTION_H
