/**
 * \file imagefiltertestrange.h <spatmod/imagefiltertestrange.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER_TESTRANGE definition.
 *
 * \if NODOC
 *
 * $Id: imagefiltertestrange.h_v 1.1 2006/12/11 15:57:53 mju Exp $
 *
 * $Log: imagefiltertestrange.h_v $
 * Revision 1.1  2006/12/11 15:57:53  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTERTESTRANGE_H
#define	INC_SPATMOD_IMAGEFILTERTESTRANGE_H

#ifndef	INC_SPATMOD_IMAGEFILTER_H
#include <spatmod/imagefilter.h>
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Filter to test pixel for containment inside (or outside) range(s).
//! Resulting image will have single sample of type UINT8 and precision 1.
//! All ranges are inclusive - values equal to range limits are included in range.
class FILTER_TESTRANGE : public FILTER {
	public:

		//! Constructor using single range of PIXEL values to test for.
		FILTER_TESTRANGE (
			STAGE *pPreviousStage,					//!< Pointer to previous stage
			bool TakeOwnership,						//!< Filter takes ownership of previous stage and will delete when needed
			const PIXEL& MinValue,					//!< Minimum value (inclusive)
			const PIXEL& MaxValue,					//!< Maximum value (inclusive)
			bool outside = false						//!< True to output 1 if outside, otherwise output is 1 if inside
			);

		//! Constructor using single range of 'double' values to test for.
		//! Input image must have single component.
		FILTER_TESTRANGE (
			STAGE *pPreviousStage,					//!< Pointer to previous stage
			bool TakeOwnership,						//!< Filter takes ownership of previous stage and will delete when needed
			double MinValue,							//!< Minimum value (inclusive)
			double MaxValue,							//!< Maximum value (inclusive)
			bool outside = false						//!< True to output 1 if outside, otherwise output is 1 if inside
			);

		//! Destructor.
		virtual ~FILTER_TESTRANGE ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		PIXEL m_PixelMin;
		PIXEL m_PixelMax;
		UINT8 **m_TableUINT8;
		bool m_Negate;

		void ProcessUINT8 (UINT16 SampleIdx, UINT8 *pTgt, const UINT8 *pSrc, UINT32 NumColumns) const;
		void ProcessUINT16 (UINT16 SampleIdx, UINT8 *pTgt, const UINT16 *pSrc, UINT32 NumColumns) const;
		void ProcessSINT16 (UINT16 SampleIdx, UINT8 *pTgt, const INT16 *pSrc, UINT32 NumColumns) const;
		void ProcessUINT32 (UINT16 SampleIdx, UINT8 *pTgt, const UINT32 *pSrc, UINT32 NumColumns) const;
		void ProcessSINT32 (UINT16 SampleIdx, UINT8 *pTgt, const INT32 *pSrc, UINT32 NumColumns) const;
		void ProcessFloat (UINT16 SampleIdx, UINT8 *pTgt, const float *pSrc, UINT32 NumColumns) const;
		void ProcessDouble (UINT16 SampleIdx, UINT8 *pTgt, const double *pSrc, UINT32 NumColumns) const;

		// STAGE overrides.
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);
		virtual ERRVALUE v_Initialize ();

		// Unimplemented
		FILTER_TESTRANGE (const FILTER_TESTRANGE&);
		FILTER_TESTRANGE& operator= (const FILTER_TESTRANGE&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTERTESTRANGE_H

