/**
 * \file imagesourcemrsid.h <spatmod/imagesourcemrsid.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::SOURCE_MRSID definition.
 *
 * \if NODOC
 * $Id: imagesourcemrsid.h_v 1.9 2011/05/20 17:01:59 mju Exp $
 *
 * $Log: imagesourcemrsid.h_v $
 * Revision 1.9  2011/05/20 17:01:59  mju
 * Add setKeepOpen.
 *
 * Revision 1.8  2010/02/19 14:19:45  mju
 * Add v_getDimensions and attempt to include in doxygen.
 *
 * Revision 1.7  2006/11/15 16:05:43  mju
 * Add readByStrip member.
 *
 * Revision 1.6  2006/11/01 23:10:24  mju
 * queryIterator.
 *
 * Revision 1.5  2006/10/19 21:25:28  mju
 * Don't need isPlatformSupported anymore.
 *
 * Revision 1.4  2006/10/19 21:24:50  mju
 * Use platform_supports_mrsid.
 *
 * Revision 1.3  2006/10/19 20:17:01  mju
 * Add isPlatformSupported.
 *
 * Revision 1.2  2006/10/19 17:21:16  mju
 * Use mrsid_reader internally.
 *
 * Revision 1.1  2006/10/18 17:49:23  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGESOURCEMRSID_H
#define	INC_SPATMOD_IMAGESOURCEMRSID_H

#ifndef	INC_MI32_PLATFORMSUPPORT_H
#include <mi32/platformsupport.h>
#endif

#if defined(PLATFORM_SUPPORTS_MRSID) || defined(GENERATING_DOXYGEN_OUTPUT)

#ifndef	INC_SPATMOD_IMAGESOURCE_H
#include <spatmod/imagesource.h>
#endif

#ifndef	INC_MI32_FILEPATH_H
#include <mi32/filepath.h>
#endif

#ifndef	GENERATING_DOXYGEN_OUTPUT
class MRSID_READER;
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Base for pipeline source.
class SOURCE_MRSID : public SOURCE {
	public:

		// Default constructor.
		explicit SOURCE_MRSID (
			const FILEPATH& FilePath
			);

		// Destructor.
		virtual ~SOURCE_MRSID ();

		//! Set whether to keep file open after initialize.
		//! Default is to keep open.
		void SetKeepOpen (bool KeepOpen)
			{ m_KeepOpen = KeepOpen; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		FILEPATH m_FilePath;
		bool m_KeepOpen;
		bool m_DoingPull;
		bool m_ReadByStrip;
		int m_NumLevels;
		mutable MRSID_READER *m_pReader;

		ERRVALUE OpenReader () const;
		void CloseReader () const;

		// BASE overrides.
		virtual ERRVALUE v_GetDimensions (DIMENSIONS& dimensions, double zoom) const;
		virtual float v_QueryIterator (const SECTION_ITERATOR& iterator) const;
		virtual ZOOMABILITY v_GetZoomAbility (SIMPLE_ARRAY<double>& ZoomsAllowed) const;

		// STAGE overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullBegin (const SECTION_ITERATOR& iterator);
		virtual ERRVALUE v_PullEnd ();
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);

		// Unimplemented
		SOURCE_MRSID (const SOURCE_MRSID&);
		SOURCE_MRSID& operator= (const SOURCE_MRSID&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// PLATFORM_SUPPORTS_MRSID
#endif	// INC_SPATMOD_IMAGESOURCEMRSID_H

