/**
 * \file spatmod/imagesourcewbmp.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::SOURCE_WBMP definition.
 *
 * \if NODOC
 * $Id: imagesourcewbmp.h_v 1.2 2011/10/11 14:47:51 mju Exp $
 *
 * $Log: imagesourcewbmp.h_v $
 * Revision 1.2  2011/10/11 14:47:51  mju
 * Remove unneeded hdrs.
 *
 * Revision 1.1  2007/02/23 14:10:16  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGESOURCEWBMP_H
#define	INC_SPATMOD_IMAGESOURCEWBMP_H

#ifndef	INC_SPATMOD_IMAGESOURCEFILE_H
#include <spatmod/imagesourcefile.h>
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Pipeline source for reading Wireless Bitmap (WBMP) images from memory or file.
class SOURCE_WBMP : public SOURCE_FILE {
	public:

		//! Construct with FILEPATH.
		explicit SOURCE_WBMP (
			const FILEPATH& FilePath
			);

		//! Construct with memory buffer (SIMPLE_ARRAY).
		//! Will take ownership of buffer.
		explicit SOURCE_WBMP (
			SIMPLE_ARRAY<UINT8>& MemBuffer	//!< Memory buffer, will be returned empty
			);

		//! Construct with memory buffer (pointer/size).
		//! Will take ownership of buffer, pointer will be returned as NULL
		SOURCE_WBMP (
			UINT8*& MemBuffer,					//!< Memory buffer, will be returned as NULL
			UINT32 BufferSize						//!< Buffer size
			);

		//! Destructor.
		virtual ~SOURCE_WBMP ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		int m_OffsetRow0;
		int m_BytesPerRow;

		// BASE overrides.
		virtual float v_QueryIterator (const SECTION_ITERATOR& iterator) const;

		// STAGE overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullBegin (const SECTION_ITERATOR& iterator);
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);

		// Unimplemented
		SOURCE_WBMP (const SOURCE_WBMP&);
		SOURCE_WBMP& operator= (const SOURCE_WBMP&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGESOURCEWBMP_H

